#include "FamilyTreeParser.h"

#include <string>
#include <iostream>
#include <fstream>

#include "TreeEntity.h"

#include "VImage.h"

//**//

bool FamilyTreeParser::parse_treedata( TreeRep *trep, QString filename )
{
	tree< TreeEntity > *_tree = &(trep->m_TreeDatastructure);
	std::string path( filename.toAscii() );

	std::string fileend = path.substr(path.size()-5, path.size()-1);

	if( fileend.compare(".tree") != 0 )
	{
		return false;
	}

	if ( !_tree->empty() ) _tree->clear();

	int next = -1; 
	//-1 not defined
	//0 root
	//1 father
	//2 mother

	TreeEntity tmp;
	
	tree< TreeEntity >::iterator topNode, rootNode, lastNode;
	topNode = (_tree->begin());

	std::ifstream in_str( path.c_str( ), std::ifstream::in );
	std::string currentLine;

	int hierarchies_tmp = 0;
	int hierarchies_max = 0;

	while(!in_str.eof())
	{
		std::getline((in_str), currentLine);

		if( currentLine.empty() && currentLine.length() > 2 )
		{
			continue;
		}

		

		//erase all tabs before the first tag
		while( currentLine[ 0 ] == ' ' )
		{
			currentLine.erase( 0, 1 );
		}

		//erase <
		currentLine.erase( 0, 1 );

		switch ( currentLine[ 0 ] )
		{
			case 'r': 
			{
				next = 0;
				break;
			}
			case 'm': 
			{
				next = 1;
				break;
			}
			case 'f': 
			{
				next = 2;

				hierarchies_tmp++;

				if ( hierarchies_max < hierarchies_tmp )
				{
					hierarchies_max = hierarchies_tmp;
				}

				break;
			}
			case 'n': //name
			{
				//root
				if ( next == 0 )
				{
					tmp = TreeEntity();

					//cut off name>
					currentLine.erase( 0, 5 );
					//cut off the tail </name>
					currentLine.erase( currentLine.length() - 7, 7 );

					tmp.name = currentLine;

					rootNode = _tree->insert( topNode, tmp );
					lastNode = rootNode;
					
					trep->m_TopNode = rootNode;

				}

				//mother
				if ( next == 1 )
				{
					tmp = TreeEntity();

					//cut off name>
					currentLine.erase( 0, 5 );
					//cut off the tail </name>
					currentLine.erase( currentLine.length() - 7, 7 );

					tmp.name = currentLine;
					tmp.sex = 1;

					lastNode = _tree->append_child( lastNode, tmp );

				}

				//father
				if ( next == 2 )
				{
					tmp = TreeEntity();

					//cut off name>
					currentLine.erase( 0, 5 );
					//cut off the tail </name>
					currentLine.erase( currentLine.length() - 7, 7 );

					tmp.name = currentLine;
					tmp.sex = 0;

					lastNode = _tree->append_child( lastNode, tmp );


				}
				
				break;
			}
			case 'i': 
			{
				//id

				//cut off id>
				currentLine.erase( 0, 3 );
				//cut off the tail </id>
				currentLine.erase( currentLine.length() - 5, 5 );

				sscanf_s(currentLine.c_str(),"%d", &(tmp.id) );

				lastNode.node->data.id = tmp.id;

				break;
			}
			case 'b': //born
			{
				//cut off born>
				currentLine.erase( 0, 5 );
				//cut off the tail </born>
				currentLine.erase( currentLine.length() - 7, 7 );
				tmp.born = currentLine;

				lastNode.node->data.born = tmp.born;

				break;
			}
			case 'd': //died
			{
				//cut off died>
				currentLine.erase( 0, 5 );
				//cut off the tail </died>
				currentLine.erase( currentLine.length() - 7, 7 );
				tmp.died = currentLine;

				lastNode.node->data.died = tmp.died;

				break;
			}
			case 'p': //picture
			{
				//cut off pic>
				currentLine.erase( 0, 4 );
				//cut off the tail </pic>
				currentLine.erase( currentLine.length() - 6, 6 );

				char buffer[ 256 ];
				sprintf_s( buffer, "./pics/%s", currentLine.c_str() );
				lastNode.node->data.m_ImagePath = buffer;
				lastNode.node->data.m_ImagePresent = true;

// 				verify_devil();
// 				verify_ilut();
				

				break;
			}
			case '/': //closing tag
			{
				//erase /
				currentLine.erase( 0, 1 );
				if( currentLine[ 0 ] == 'm' || currentLine[ 0 ] == 'f')
				{
					//mother or father
					lastNode = lastNode.node->parent;

					if( currentLine[ 0 ] == 'm' )
					{
						hierarchies_tmp--;
							
					}
				}
				break;
			}
			
		}
	}

	in_str.close();

	trep->m_iNumHierarchies = hierarchies_max;

	return true;

}

bool FamilyTreeParser::parse_relationdata( std::vector< strRelationshipPair > *trel, QString filename )
{

	std::string path( filename.toAscii() );

	if( path.length() < 4 ) return false;

	std::string fileend = path.substr(path.size()-4, path.size()-1);

	if( fileend.compare(".rel") != 0 )
	{
		return false;
	}

	if ( !trel->empty() ) trel->clear();


	std::ifstream in_str( path.c_str( ), std::ifstream::in );
	std::string currentLine;

	while(!in_str.eof())
	{
		std::getline((in_str), currentLine);

		if( currentLine.empty() && currentLine.length() > 2 )
		{
			continue;
		}

		//erase all tabs before the first tag
		while( currentLine[ 0 ] == ' ' )
		{
			currentLine.erase( 0, 1 );
		}

		switch ( currentLine[ 0 ] )
		{
			case '/': 
			{
				// comment
				break;
			}

			case '<': 
			{
				// starting tag, also not interesting
				break;
			}
			default :
			{
				int id1, id2;

				sscanf_s(currentLine.c_str(),"%d%d", &id1, &id2 );	

				strRelationshipPair rp( id1, id2 );
				trel->push_back( rp );
			}

			
		}
	}

	return true;
}