#include "QBackgroundCanvas.h"




QBackgroundCanvas::QBackgroundCanvas(QWidget* parent /*= 0*/) : QObject(parent), QGLWidget(parent), m_VolCanvasColor( NULL )
{

	glInit ();
}


QBackgroundCanvas::~QBackgroundCanvas()
{
	
}

void QBackgroundCanvas::setObjectName(const QString &name)
{
}

void QBackgroundCanvas::setVolBackColorPtr(VVector *v)
{
	m_VolCanvasColor = v;

	/*m_VolCanvasColor->setX(0.0f);
	m_VolCanvasColor->setY(0.0f);
	m_VolCanvasColor->setZ(0.0f);*/
}

void QBackgroundCanvas::initializeGL()
{
	this->makeCurrent();
    // Set up the rendering context, define display lists etc.:
    //...
	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);
    //...

	paintGL();
}

void QBackgroundCanvas::resizeGL( int w, int h )
{
	this->makeCurrent();

    glViewport(0, 0, w, h);
	gluOrtho2D(0, w, h, 0);
	
}

void QBackgroundCanvas::paintGL()
{	
	if( ! m_VolCanvasColor ) return;

	/*this*/makeCurrent();
	glClearColor( m_VolCanvasColor->getX(), m_VolCanvasColor->getY(), m_VolCanvasColor->getZ(), 1.0f );
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
    glEnable(GL_DEPTH_TEST);

	//glEnable(GL_ALPHA_TEST);
	//glAlphaFunc(GL_GREATER, 0.5f);
    // draw the scene:
    glPushAttrib(GL_ALL_ATTRIB_BITS);
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	glLoadIdentity();									// Reset The Current Modelview Matrix
	
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glPopAttrib();
}

void QBackgroundCanvas::redraw()
{
	makeCurrent();
	repaint();
	swapBuffers();
}
void QBackgroundCanvas::paintEvent( QPaintEvent *e )
{
	QGLWidget::paintEvent(e);
};

void QBackgroundCanvas::mousePressEvent ( QMouseEvent * e )
{			
	//const VMouseEvent me(getMouseEvent(e));
	QColor qcolor = QColorDialog::getColor();
	VVector color( ( float )qcolor.red()/255.0f, ( float )qcolor.green()/255.0f, ( float )qcolor.blue(  )/255.0f );
	
	m_VolCanvasColor->setX( color.getX() );
	m_VolCanvasColor->setY( color.getY() );
	m_VolCanvasColor->setZ( color.getZ() );

	redraw();
	
};

void QBackgroundCanvas::mouseReleaseEvent ( QMouseEvent * e )
{
	//const VMouseEvent me(getMouseEvent(e));

};
	
void QBackgroundCanvas::mouseMoveEvent ( QMouseEvent * e )
{
	const VMouseEvent me(getMouseEvent(e));

	// v is the position of the mouse
	// cursor over canvas: v ranges in R, from -1 to 1 in both directions, left bottom corner is -1,-1
	// cursor n.o. canvas: if mouse was pressed before cursor left canvas, position is outside the range R 
	const VVector v  = me.getPosition();

	//std::cout << "VolRend: " << v << std::endl;

};

void QBackgroundCanvas::mouseDoubleClickEvent ( QMouseEvent * e )
{
	
};

void QBackgroundCanvas::keyPressEvent ( QKeyEvent * e )
{
	//const VKeyboardEvent ke(getKeyboardEvent(e));

};

void QBackgroundCanvas::keyReleaseEvent ( QKeyEvent * e )
{
	//const VKeyboardEvent ke(getKeyboardEvent(e));
	
};

const VMouseEvent QBackgroundCanvas::getMouseEvent (QMouseEvent *e)
{
	const VVector vecPosition( (2.0f * float(e->x()) - float(width()))  / float(width()),
		                      ( float(height()) - 2.0f * float(e->y())) / float(height()),
							   0.0f);
	const int iButton = 
				((e->button() == Qt::LeftButton)	? VMouseEvent::BUTTON_LEFT	 : 
				((e->button() == Qt::MidButton)		? VMouseEvent::BUTTON_MIDDLE : 
				((e->button() == Qt::RightButton)	? VMouseEvent::BUTTON_RIGHT	 : VMouseEvent::BUTTON_NONE)));

	const int iStateLeft	= (e->buttons () & Qt::LeftButton)	? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;
	const int iStateMiddle	= (e->buttons () & Qt::MidButton)	? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;
	const int iStateRight	= (e->buttons () & Qt::RightButton) ? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;

	const int iModifiers =   ((e->modifiers()   & Qt::ShiftModifier)   ? VKeyboardEvent::MODIFIER_SHIFT : 0) 
						   | ((e->modifiers()	& Qt::ControlModifier) ? VKeyboardEvent::MODIFIER_CTRL  : 0) 
						   | ((e->modifiers()	& Qt::AltModifier)     ? VKeyboardEvent::MODIFIER_ALT   : 0);

	return VMouseEvent(vecPosition,iButton,iStateLeft,iStateMiddle,iStateRight,iModifiers);

};

const VKeyboardEvent QBackgroundCanvas::getKeyboardEvent (QKeyEvent *e)
{
	const int iKey = e->key();

	const int iModifiers =	  ((e->modifiers() & Qt::ShiftModifier)		? VKeyboardEvent::MODIFIER_SHIFT : 0) 
							| ((e->modifiers() & Qt::ControlModifier)	? VKeyboardEvent::MODIFIER_CTRL	 : 0) 
							| ((e->modifiers() & Qt::AltModifier)		? VKeyboardEvent::MODIFIER_ALT	 : 0);

	return VKeyboardEvent(iKey,iModifiers);

};