#ifndef QBACKGROUNDCANVAS_H_
#define QBACKGROUNDCANVAS_H_

#include <Qt/qobject.h>

#include <QtGui/QWidget>
#include <QtOpenGL/QGLWidget>
#include <QtGui/QKeyEvent>
#include <QtGui/QMouseEvent>
#include <QtGui/QColorDialog>

#include "VEvent.h"
#include "VVector.h"

/**
 * class for rendering the color of the background, when clicked it provides the user with a color picker dialog
 */
class QBackgroundCanvas : public QObject, public QGLWidget
{
	Q_OBJECT

protected:

	/**
	 * method to initialize opengl
	 */
	virtual void initializeGL();

	/**
	 * method for repainting. called in every refresh
	 */
    virtual void paintGL();

public:
	/**
	 * constructor with given QWidget Pointer
	 */
	QBackgroundCanvas(QWidget* parent = 0);
	
	/**
	 * default destructor
	 */
	virtual ~QBackgroundCanvas();

	/**
	 * sets the name of the Object
	 * @param name reference to a QString containing the name
	 */
	void setObjectName(const QString &name);
	
	/**
	 * method for setting the pointer to the volumerenderers backgroundcolor
	 * @param v Pointer to VVector
	 */
	void setVolBackColorPtr(VVector *v);

	/**
	 * method called when window is resized
	 * @param w width of widget
	 * @param h height of widget
	 */
    virtual void resizeGL( int w, int h );
	
	//events
	/**
	 * event callback for the paint event
	 * @param e pointer to QPaintEvent
	 */
	virtual void paintEvent( QPaintEvent *e );
	
	/**
	 * event callback for the mousepressevent
	 * @param e Pointer to QMouseEvent
	 */
	virtual void mousePressEvent ( QMouseEvent * e );

	/**
	* event callback for the mousereleaseevent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseReleaseEvent ( QMouseEvent * e );

	/**
	* event callback for the mousemoveevent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseMoveEvent ( QMouseEvent * e );

	/**
	* event callback for the mousedoubleclickevent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseDoubleClickEvent ( QMouseEvent * e );

	/**
	* event callback for the keypressevent
	* @param e Pointer to QKeyEvent
	*/
	virtual void keyPressEvent ( QKeyEvent * e );
	
	/**
	* event callback for the keyreleaseevent
	* @param e Pointer to QKeyEvent
	*/
	virtual void keyReleaseEvent ( QKeyEvent * e );

	/**
	* Method for creating the current Mousevent
	* @param e Pointer to QMouseEvent
	* @return VMouseEvent
	*/
	const VMouseEvent getMouseEvent (QMouseEvent *e);

	/**
	 * Method for creating the Keybard event
	 * @param e QKeyEvent
	 * @return VKeyboardEvent
	 */
	const VKeyboardEvent getKeyboardEvent (QKeyEvent *e);

	/**
	 * method called when Canvas is readrawn.
	 * Calls paintGl
	 */
	virtual void redraw();

private:

	VVector *m_VolCanvasColor;			/**< VVector Pointer pointing at the Color of the mainrenderercanvas. */



};
#endif
