#ifndef QDOUBLECANVASCOLOR_H_
#define QDOUBLECANVASCOLOR_H_

#include <Qt/qobject.h>

#include <QtGui/QWidget>
#include <QtOpenGL/QGLWidget>
#include <QtGui/QKeyEvent>
#include <QtGui/QMouseEvent>
#include <QtGui/QColorDialog>

#include "VEvent.h"
#include "VVector.h"

/**
* class for rendering the color of the background, when clicked it provides the user with a color picker dialog
*/
class QDoubleCanvasColor : public QObject, public QGLWidget
{
	Q_OBJECT

protected:

	/**
	* method to initialize opengl
	*/
	virtual void initializeGL();

	/**
	* method for repainting. called in every refresh
	*/
	virtual void paintGL();

public:
	/**
	* constructor with given QWidget Pointer
	*/
	QDoubleCanvasColor(QWidget* parent = 0);

	/**
	* default destructor
	*/
	virtual ~QDoubleCanvasColor();

	/**
	* sets the name of the Object
	* @param name reference to a QString containing the name
	*/
	void setObjectName(const QString &name);

	/**
	* method for setting the pointer to the volumerenderers backgroundcolor No1
	* @param v Pointer to VVector
	*/
	void setVolBackColorPtr_1(VVector *v);

	/**
	* method for setting the pointer to the volumerenderers backgroundcolor No2
	* @param v Pointer to VVector
	*/
	void setVolBackColorPtr_2(VVector *v);

	/**
	* method for setting tooltip No1
	* @param s is the tooltip string
	*/
	void setTooltip_1(QString s);

	/**
	* method for setting the tooltip No2
	* @param s is the tooltip string
	*/
	void setTooltip_2(QString s);

	/**
	* method called when window is resized
	* @param w width of widget
	* @param h height of widget
	*/
	virtual void resizeGL( int w, int h );

	//events
	/**
	* event callback for the paint event
	* @param e pointer to QPaintEvent
	*/
	virtual void paintEvent( QPaintEvent *e );

	/**
	* event callback for the mousepressevent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mousePressEvent ( QMouseEvent * e );

	/**
	* event callback for the mousereleaseevent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseReleaseEvent ( QMouseEvent * e );

	/**
	* event callback for the mousemoveevent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseMoveEvent ( QMouseEvent * e );

	/**
	* event callback for the mousedoubleclickevent
	* @param e Pointer to QMouseEvent
	*/
	virtual void mouseDoubleClickEvent ( QMouseEvent * e );

	/**
	* event callback for the keypressevent
	* @param e Pointer to QKeyEvent
	*/
	virtual void keyPressEvent ( QKeyEvent * e );

	/**
	* event callback for the keyreleaseevent
	* @param e Pointer to QKeyEvent
	*/
	virtual void keyReleaseEvent ( QKeyEvent * e );

	/**
	* Method for creating the current Mousevent
	* @param e Pointer to QMouseEvent
	* @return VMouseEvent
	*/
	const VMouseEvent getMouseEvent (QMouseEvent *e);

	/**
	* Method for creating the Keybard event
	* @param e QKeyEvent
	* @return VKeyboardEvent
	*/
	const VKeyboardEvent getKeyboardEvent (QKeyEvent *e);

	/**
	* method called when Canvas is readrawn.
	* Calls paintGl
	*/
	virtual void redraw();

private:

	VVector *m_ptrCanvasColor_1;			/**< VVector Pointer pointing at the Color of the mainrenderercanvas. */
	VVector *m_ptrCanvasColor_2;			/**< VVector Pointer pointing at the Color of the mainrenderercanvas. */

	bool m_bColorSet_1;						/**< Is true if the 1st color was already set. */
	bool m_bColorSet_2;						/**< Is true if the 2nd color was already set. */

	QString m_qstrTooltip_1;				/**< Contains the tooltip for the left part of the canvas, shows up if the mouse is over. */
	QString m_qstrTooltip_2;				/**< Contains the tooltip for the right part of the canvas, shows up if the mouse is over. */

};
#endif
