#ifndef QFOTOCANVAS_H_
#define QFOTOCANVAS_H_

#include <Qt/qobject.h>

#include <Qt/qobject.h>

#include <QtGui/QWidget>
#include <QtOpenGL/QGLWidget>

#include "VVector.h"
#include "VImage.h"

#include "TreeRep.h"

/**
* class for rendering the selected pictures of an entity
*/
class QFotoCanvas : public QObject, public QGLWidget
{
	Q_OBJECT

protected:

	/**
	* method to initialize opengl
	*/
	virtual void initializeGL();

	/**
	* method for repainting. called in every refresh
	*/
	virtual void paintGL();

public:
	/**
	* constructor with given QWidget Pointer
	*/
	QFotoCanvas(QWidget* parent = 0);

	/**
	* default destructor
	*/
	virtual ~QFotoCanvas();

	/**
	* sets the name of the Object
	* @param name reference to a QString containing the name
	*/
	void setObjectName(const QString &name);

	/**
	* method for setting the pointer to the volumerenderers backgroundcolor
	* @param v Pointer to VVector
	*/
	void setVolBackColorPtr(VVector *v);

	/**
	* method for setting the pointer to the texture handle of the foto
	* @param v Pointer to the texture handle
	*/
	void setTexHandlePtr(VImage *h);

	/**
	* method called when window is resized
	* @param w width of widget
	* @param h height of widget
	*/
	virtual void resizeGL( int w, int h );

	//events
	/**
	* event callback for the paint event
	* @param e pointer to QPaintEvent
	*/
	virtual void paintEvent( QPaintEvent *e );

	/**
	* method called when Canvas is readrawn.
	* Calls paintGl
	*/
	virtual void redraw();

	TreeRep *m_TreeRep;					/**< Encasules the tree data structure and its properties. */

private:

	VVector *m_ptrCanvasColor;			/**< VVector Pointer pointing at the Color of the mainrenderercanvas. */
	VImage *m_TexHandle;				/**< Handle to the Image containing the foto which is rendered. */
	bool m_bColorSet;					/**< Is true if the color was already set. */

};
#endif