#ifndef QMAIN_H_
#define QMAIN_H_

#include "glew.h"
#include <QtOpenGL/QtOpenGL>

#include "./Ui_QMain.h"
#include <Qt/qtimer.h>

#include "FamilyTreeParser.h"

#include "TreeRep.h"

/**
 * main QT Class handling the whole window related Stuff
 */
class QMain : public QMainWindow, protected Ui_EdgeBundles
{
	Q_OBJECT
public:

	/**
	 * constructor with given QWidget Pointer and additional Flags
	 * @param parent Pointer to Parent Widget.
	 * @param flags additional Flags for window creation
	 */
	QMain(QWidget* parent = 0, Qt::WFlags flags = 0);

	/**
	 * default destructor
	 */
	virtual ~QMain();

public slots:

	/**
	 * watches for any events
	 */
	void watch();

	/**
	 * handles redrawing of the whole window
	 */
	void redraw();

	/**
	 * opens a file dialog to load a dataset
	 */
	void openDataFile();

	/**
	* opens a file dialog to load a relation datafile
	*/
	void openRealtionsFile();

	/**
	 * close action
	 */
	void close();

	/**
	 * value of the radial tree button changed
	 * @param val is the value of the button
	 */
	void radtreeb_changed( bool val );

	/**
	 * value of the inverted tree button changed
	 * @param val is the value of the button
	 */
	void invradtreeb_changed( bool val );

	/**
	 * value of the rooted tree button changed
	 * @param val is the value of the button
	 */
	void roottreeb_changed( bool val );

	/**
	 * value of the inverted rooted tree button changed
	 * @param val is the value of the button
	 */
	void invroottreeb_changed( bool val );

	/**
	 * value of the ballon tree button changed
	 * @param val is the value of the button
	 */
	void baltreeb_changed( bool val );

	/**
	 * value of the beta slider has changed
	 * @param val is the value of the slider
	 */
	void betaval_changed( int val );

	/**
	 * value of the alpha slider has changed
	 * @param val is the value of the slider
	 */
	void alphaval_changed( int val );

	/**
	 * value of the emphasize slider has changed
	 * @param val is the value of the slider
	 */
	void emphval_changed( int val );

	/**
	 * value of selection check box
	 * @param val is the status of the check box
	 */
	void select_splines( bool val );

private:

	QTimer *pTimer;						/**< Timer for event handling. */
	QTimer *pTimer_redraw;				/**< Timer for redrawing. has other granularity as pTimer. */
	
	VVector m_BackgroundColor;			/**< VVector containing the BackgroundColor of the rendererCanvas. */

	VVector m_GraphColor_1;				/**< VVector containing the line color of the graphs. */
	VVector m_GraphColor_2;				/**< VVector containing the nodes color of the graph. */

	VVector m_SplineColor_1;			/**< VVector containing the splines starting color. */
	VVector m_SplineColor_2;			/**< VVector containing the splines end color. */

	int layout_state;					/**< int storing the coded state of the layout. */
	bool splines_selstate;				/**< bool storing the state (on/off) of enabling the splines selection. */
	
	int w_sizex;						/**< int storing the size of the window in x direction. */
	int w_sizey;						/**< int storing the size of the window in y direction. */

	int pixoffsetx_rcanvas;				/**< int storing the offset of the canvas. */
	int pixoffsety_rcanvas;				/**< int storing the offset of the canvas. */

	int pixposx_rcanvas;				/**< int storing the position of the canvas. */
	int pixposy_rcanvas;				/**< int storing the position of the canvas. */

	TreeRep		m_Tree;					/**< a tree abstract data structure containing the hierarchical relations*/
	bool		data_present;			/**< true if there is data file loaded*/
	bool		rel_present;			/**< true if there is relation file loaded*/

	std::vector< strRelationshipPair > relations; /**< a vector containing all relationships pairs*/

};
#endif