#include "Spline.h"

#include "glew.h"
#include <gl/gl.h>			
#include <gl/glu.h>	

#include <iostream>
#include <vector>
#include <algorithm>

// TODO: inv. radial tree...how to connect?

bool segmentIntersection( float *line1, float *line2 )
{
	float x1,x2,x3,x4;
	float y1,y2,y3,y4;

	x1 = line1[ 0 ];
	x2 = line1[ 2 ];
	x3 = line2[ 0 ];
	x4 = line2[ 2 ];

	y1 = line1[ 1 ];
	y2 = line1[ 3 ];
	y3 = line2[ 1 ];
	y4 = line2[ 3 ];

	float ua = ( ( x4 - x3 ) * ( y1 - y3 ) - ( y4 - y3 ) * ( x1 - x3 ) ) /
			   ( ( y4 - y3 ) * ( x2 - x1 ) - ( x4 - x3 ) * ( y2 - y1 ) );

	if ( ua > 1.0 || ua < 0.0 ) return false;

	float ub = ( ( x2 - x1 ) * ( y1 - y3 ) - ( y2 - y1 ) * ( x1 - x3 ) ) /
			   ( ( y4 - y3 ) * ( x2 - x1 ) - ( x4 - x3 ) * ( y2 - y1 ) );

	if ( ub > 1.0 || ub < 0.0 ) return false;

	return true;
}

double factorial( double n )
{
	if ( n > 52 )
	{
		std::cerr<< "Cannot compute factorial of numbers greater than 52 )" << std::endl;
	}
	double fac = 1.0;

	for( int i = 1; i <= n; i++ )
	{
		fac *= i;
	}

	return fac;

}

void Spline::draw(  )
{
	if( !m_bStatus ) 
	{
		return;
	}

	glLineWidth( 2.0f );

	//glBlendFuncSeparateEXT(GL_SRC_COLOR, GL_DST_COLOR, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	
	//glBlendEquationEXT(GL_MIN_EXT);
	//glBlendEquationEXT(GL_MAX_EXT);
	glAlphaFunc(GL_GREATER,0.1);
	glEnable(GL_ALPHA_TEST)	;


	GLUnurbs* nurb = gluNewNurbsRenderer();
	gluNurbsProperty(nurb, GLU_SAMPLING_METHOD, GLU_PATH_LENGTH);
	gluNurbsProperty(nurb, GLU_SAMPLING_TOLERANCE, 10.0);

	//straighten control points
	std::vector<GLfloat> controlpoints;
	controlpoints.clear();
	for (unsigned int i =0; i< points.size() ;i++)
	{
		controlpoints.push_back((GLfloat)m_fBeta*points[i]->cart_x + (1-m_fBeta)*(points[0]->cart_x + i/(points.size()-1)*(points[points.size()-1]->cart_x - points[0]->cart_x)));
		controlpoints.push_back((GLfloat)m_fBeta*points[i]->cart_y + (1-m_fBeta)*(points[0]->cart_y + i/(points.size()-1)*(points[points.size()-1]->cart_y - points[0]->cart_y)));
		controlpoints.push_back((GLfloat)0);
	}

	// approximate the length of the curve by computing linear length between controlpoints
	float length = 0;
	std::vector<float> lengths;	//for storing partial lengths for each controlpoint
	lengths.clear();
	for (unsigned int i = 0; i < controlpoints.size()-3; i+=3)
	{
		// current controlpoint
		float x1 = controlpoints[i-i%3];	
		float y1 = controlpoints[i-i%3+1];
		// next controlpoint
		float x2 = controlpoints[i+3-i%3];
		float y2 = controlpoints[i+3-i%3+1];
		// store partial length for current controlpoint
		lengths.push_back(length);
		// length by euclidean distance...less accurate approximation could be used to optimize
		length += sqrt(pow(x1-x2,2) + pow(y1-y2,2));
	}
	// store length for last controlpoint
	lengths.push_back(length);

	std::vector<GLfloat> color;
	color.clear();
	for (unsigned int i =0; i< points.size() ;i++)
	{
		float r_start = m_vColorStart->getX();
		float g_start = m_vColorStart->getY();
		float b_start = m_vColorStart->getZ();
		float r_end = m_vColorEnd->getX();
		float g_end = m_vColorEnd->getY();
		float b_end = m_vColorEnd->getZ();

		float factor = m_fEmphasize * ( (float)points.size() - 3.0f ) / ( 3.0f - (float)m_LongestSpline ) + 1.0;
		factor = min( factor, 1.0 );
		factor = max( factor, 0.1 );

			//( max( 1.0f -( (float)points.size() - 5.0f)/(float) m_LongestSpline, 0.3f ), 1.0f );
		//float emphasized = m_fEmphasize * factor; //exp( ( 1.0f - m_fEmphasize ) * log ( factor ) );
		float alpha = m_fAlpha * factor;
		// colors have to be interpolated for each control point
		// according to ratio of the length of the curve from current point to whole length
		color.push_back((GLfloat) (r_start + (r_end-r_start)*(lengths[i]/(length))));// * alpha);
		color.push_back((GLfloat) (g_start + (g_end-g_start)*(lengths[i]/(length))));// * alpha);
		color.push_back((GLfloat) (b_start + (b_end-b_start)*(lengths[i]/(length))));// * alpha);
		color.push_back(alpha);
		
	}

	//// show straightened control points
	//for( unsigned int i=0; i<controlpoints.size(); i++)
	//{
	//	
	//	glColor3f( 1,0,0);
	//	glPointSize( 10.0f );
	//	glBegin( GL_POINTS );
	//		glVertex2f( controlpoints[i-i%3], controlpoints[i-i%3+1] );
	//	glEnd();
	//	glColor3f( 1,1,1);
	//}

	// draw curve
	// with less points, the degree is smaller...
	if (points.size() < 3)
	{
		//glColor3fv( m_vColorEnd->getPtr() );
		
		GLfloat knots[4] = { 0.0, 0.0, 1.0, 1.0 };
		
			gluBeginCurve(nurb);
				gluNurbsCurve(nurb,
							4, knots,
							4,
							&color[0],
							2,
							GL_MAP1_COLOR_4);
				gluNurbsCurve(nurb,
								4, knots,
								3,
								&controlpoints[0],
								2,
								GL_MAP1_VERTEX_3);
			gluEndCurve(nurb);
		

	}
	else if (points.size()==3)
	{
		//glColor3fv( m_vColorEnd->getPtr() );
		
		GLfloat knots[6] = { 0.0, 0.0,0.0, 1.0, 1.0, 1.0 };
		
			gluBeginCurve(nurb);
				gluNurbsCurve(nurb,
							6, knots,
							4,
							&color[0],
							3,
							GL_MAP1_COLOR_4);
				gluNurbsCurve(nurb,
								6, knots,
								3,
								&controlpoints[0],
								3,
								GL_MAP1_VERTEX_3);
			gluEndCurve(nurb);
		

	}
	else
	{

		//glColor3fv( m_vColorStart->getPtr() );
		int order = 4; // curve degree +1
		// number of knot vector = number of control points + degree + 1
		int knotCount = points.size()+ order;

		// create knots
		unsigned int i=0;
		std::vector<GLfloat> knots;
		knots.clear();

		// fill open uniform knot vector
		knots.push_back((GLfloat)0);
		knots.push_back((GLfloat)0);
		knots.push_back((GLfloat)0);
		for (i=0; i < knotCount-7; i++)
		{
			knots.push_back((GLfloat)i);
		}
		knots.push_back((GLfloat)i);
		knots.push_back((GLfloat)i);
		knots.push_back((GLfloat)i);
		knots.push_back((GLfloat)i);

		 //for (i=0; i<knotCount; i++)
			// std::cout << " " << knots[i];

		 //std::cout<<"\n";

		// draw curve
		gluBeginCurve(nurb);
			gluNurbsCurve(nurb,
							knotCount, &knots[0],
							4,
							&color[0],
							order,
							GL_MAP1_COLOR_4);
			gluNurbsCurve(nurb,
							knotCount, &knots[0],
							3,
							&controlpoints[0],
							order,
							GL_MAP1_VERTEX_3);
		gluEndCurve(nurb);

	}
	gluDeleteNurbsRenderer(nurb);
}

void Spline::drawInvRooted(  )
{
	if( !m_bStatus ) return;

	glLineWidth( 2.0f );

	GLUnurbs* nurb = gluNewNurbsRenderer();
	gluNurbsProperty(nurb, GLU_SAMPLING_METHOD, GLU_PATH_LENGTH);
	gluNurbsProperty(nurb, GLU_SAMPLING_TOLERANCE, 10.0);

	//straighten control points
	std::vector<GLfloat> controlpoints;
	controlpoints.clear();
	for (unsigned int i =0; i< points.size() ;i++)
	{
		GLfloat x = (GLfloat)m_fBeta*points[i]->cart_x + (1-m_fBeta)*(points[0]->cart_x + i/(points.size()-1)*(points[points.size()-1]->cart_x - points[0]->cart_x));
		GLfloat y = (GLfloat)m_fBeta*points[i]->cart_y + (1-m_fBeta)*(points[0]->cart_y + i/(points.size()-1)*(points[points.size()-1]->cart_y - points[0]->cart_y));

		//flip y
		if (i>1 && i <points.size()-1)
		{
			y = points[0]->cart_y - abs( y - points[0]->cart_y );
		}

		controlpoints.push_back(x);
		controlpoints.push_back(y);
		controlpoints.push_back((GLfloat)0);
	}

	
	// approximate the length of the curve by computing linear length between controlpoints
	float length = 0;
	std::vector<float> lengths;	//for storing partial lengths for each controlpoint
	lengths.clear();
	for (unsigned int i = 0; i < controlpoints.size()-3; i+=3)
	{
		// current controlpoint
		float x1 = controlpoints[i-i%3];	
		float y1 = controlpoints[i-i%3+1];
		// next controlpoint
		float x2 = controlpoints[i+3-i%3];
		float y2 = controlpoints[i+3-i%3+1];
		// store partial length for current controlpoint
		lengths.push_back(length);
		// length by euclidean distance...less accurate approximation could be used to optimize
		length += sqrt(pow(x1-x2,2) + pow(y1-y2,2));
	}
	// store length for last controlpoint
	lengths.push_back(length);

	std::vector<GLfloat> color;
	color.clear();
	for (unsigned int i =0; i< points.size() ;i++)
	{
		float r_start = m_vColorStart->getX();
		float g_start = m_vColorStart->getY();
		float b_start = m_vColorStart->getZ();
		float r_end = m_vColorEnd->getX();
		float g_end = m_vColorEnd->getY();
		float b_end = m_vColorEnd->getZ();

		float factor = m_fEmphasize * ( (float)points.size() - 3.0f ) / ( 3.0f - (float)m_LongestSpline ) + 1.0;
		factor = min( factor, 1.0 );
		factor = max( factor, 0.1 );

		//( max( 1.0f -( (float)points.size() - 5.0f)/(float) m_LongestSpline, 0.3f ), 1.0f );
		//float emphasized = m_fEmphasize * factor; //exp( ( 1.0f - m_fEmphasize ) * log ( factor ) );
		float alpha = m_fAlpha * factor;

		// colors have to be interpolated for each control point
		// according to ratio of the length of the curve from current point to whole length
		color.push_back((GLfloat) (r_start + (r_end-r_start)*(lengths[i]/(length))));// * alpha);
		color.push_back((GLfloat) (g_start + (g_end-g_start)*(lengths[i]/(length))));// * alpha);
		color.push_back((GLfloat) (b_start + (b_end-b_start)*(lengths[i]/(length))));// * alpha);
		color.push_back(alpha);
	}

	// draw curve
	// with less points, the degree is smaller...
	if (points.size() < 3)
	{
		//glColor3fv( m_vColorEnd->getPtr() );
		
		GLfloat knots[4] = { 0.0, 0.0, 1.0, 1.0 };
		
			gluBeginCurve(nurb);
				gluNurbsCurve(nurb,
							4, knots,
							4,
							&color[0],
							2,
							GL_MAP1_COLOR_4);
				gluNurbsCurve(nurb,
								4, knots,
								3,
								&controlpoints[0],
								2,
								GL_MAP1_VERTEX_3);
			gluEndCurve(nurb);
		

	}
	else if (points.size()==3)
	{
		//glColor3fv( m_vColorEnd->getPtr() );
		
		GLfloat knots[6] = { 0.0, 0.0,0.0, 1.0, 1.0, 1.0 };
		
			gluBeginCurve(nurb);
				gluNurbsCurve(nurb,
							6, knots,
							4,
							&color[0],
							3,
							GL_MAP1_COLOR_4);
				gluNurbsCurve(nurb,
								6, knots,
								3,
								&controlpoints[0],
								3,
								GL_MAP1_VERTEX_3);
			gluEndCurve(nurb);
		

	}
	else
	{

		//glColor3fv( m_vColorStart->getPtr() );
		int order = 4; // curve degree +1
		// number of knot vector = number of control points + degree + 1
		int knotCount = points.size()+ order;

		// create knots
		unsigned int i=0;
		std::vector<GLfloat> knots;
		knots.clear();

		// fill open uniform knot vector
		knots.push_back((GLfloat)0);
		knots.push_back((GLfloat)0);
		knots.push_back((GLfloat)0);
		for (i=0; i < knotCount-7; i++)
		{
			knots.push_back((GLfloat)i);
		}
		knots.push_back((GLfloat)i);
		knots.push_back((GLfloat)i);
		knots.push_back((GLfloat)i);
		knots.push_back((GLfloat)i);

		 //for (i=0; i<knotCount; i++)
			// std::cout << " " << knots[i];

		 //std::cout<<"\n";

		// draw curve
		gluBeginCurve(nurb);
			gluNurbsCurve(nurb,
							knotCount, &knots[0],
							4,
							&color[0],
							order,
							GL_MAP1_COLOR_4);
			gluNurbsCurve(nurb,
							knotCount, &knots[0],
							3,
							&controlpoints[0],
							order,
							GL_MAP1_VERTEX_3);
		gluEndCurve(nurb);

	}
	gluDeleteNurbsRenderer(nurb);
}

bool Spline::hasIntersectionSpecial( float *line )
{

	//straighten control points
	std::vector<GLfloat> controlpoints;
	controlpoints.clear();
	for (unsigned int i =0; i< points.size() ;i++)
	{
		GLfloat x = (GLfloat)m_fBeta*points[i]->cart_x + (1-m_fBeta)*(points[0]->cart_x + i/(points.size()-1)*(points[points.size()-1]->cart_x - points[0]->cart_x));
		GLfloat y = (GLfloat)m_fBeta*points[i]->cart_y + (1-m_fBeta)*(points[0]->cart_y + i/(points.size()-1)*(points[points.size()-1]->cart_y - points[0]->cart_y));

		//flip y
		if (i>1 && i <points.size()-1)
		{
			y = points[0]->cart_y - abs( y - points[0]->cart_y );
		}

		controlpoints.push_back(x);
		controlpoints.push_back(y);
		controlpoints.push_back((GLfloat)0);
	}

	bool intersects = false;

	// draw curve
	// with less points, the degree is smaller...
	if (points.size() < 3)
	{
		float knots[4] = { 0.0, 0.0, 1.0, 1.0 };

		VVector p1 = VVector( controlpoints[ 0 ], controlpoints[ 1 ], 0.0f );
		VVector p2 = VVector( controlpoints[ 3 ], controlpoints[ 4 ], 0.0f );
		rasterized.push_back( p1 );
		rasterized.push_back( p2 );	
		
		float _line[ 4 ];
		_line[ 0 ] = p1.getX();
		_line[ 1 ] = p1.getY();
		_line[ 2 ] = p2.getX();
		_line[ 3 ] = p2.getY();

		if ( segmentIntersection( line , _line ) ) 
		{
			return true;
		}
		
		
	}
	else if (points.size()==3)
	{
		
		VVector sum;
		VVector p;
		int n = points.size() - 1;
		rasterized.clear();

		/*glColor4f( 1.0f, 0.0f, 0.0f, 1.0f );
		glBegin( GL_LINE_STRIP );*/
		for ( float t = 0.0f; t <= 1.0; t += (float)0.01f  )
		{
			sum = VVector( );

			if ( t == 0.0 )
			{
				sum = VVector( controlpoints[ 0 ], controlpoints[ 1 ], controlpoints[ 2 ] );
			}
			else if ( t == 1.0 )
			{
				sum = VVector( controlpoints[ 3*n ], controlpoints[ 3*n + 1 ], controlpoints[ 3*n + 2 ] );
			}
			else 
			{

				for ( int i = 0; i <= n; i ++ )
				{
					p = VVector( controlpoints[ 3*i ], controlpoints[ 3*i + 1 ], controlpoints[ 3*i + 2 ] );
			
					double binome = factorial( (double)n ) / ( factorial( (double)i ) * factorial( (double)( n -i ) ));
					double factor1 = exp( i * log( t ) );
					double factor2 = exp( ( n - i ) * log( 1.0 - t ) ); 
					float factor = (float) ( binome * factor1 * factor2 );
					p *= factor;
					sum += p;
					
				}
			}

			//glVertex3fv( sum.getPtr() );
			rasterized.push_back( sum );

			if ( rasterized.size() > 1 )
			{
				VVector p1 = rasterized[ rasterized.size() - 2 ];
				VVector p2 = rasterized[ rasterized.size() - 1 ];
				float _line[ 4 ];
				_line[ 0 ] = p1.getX();
				_line[ 1 ] = p1.getY();
				_line[ 2 ] = p2.getX();
				_line[ 3 ] = p2.getY();

				if ( segmentIntersection( line , _line ) ) 
				{
					return true;
				}
			}
			
		}
		//glEnd();
		
	}
	else
	{

		//glColor3fv( m_vColorStart->getPtr() );
		int order = 4; // curve degree +1
		// number of knot vector = number of control points + degree + 1
		int knotCount = points.size()+ order;

		// create knots
		unsigned int i=0;
		std::vector<float> knots;
		knots.clear();

		// fill open uniform knot vector
		knots.push_back((float)0);
		knots.push_back((float)0);
		knots.push_back((float)0);
		for (i=0; i < knotCount-7; i++)
		{
			knots.push_back((float)i / (float)(knotCount - 7.0f ));
		}
		knots.push_back((float)i / (float)( knotCount - 7.0f ));
		knots.push_back((float)i / (float)( knotCount - 7.0f ));
		knots.push_back((float)i / (float)( knotCount - 7.0f ));
		knots.push_back((float)i / (float)( knotCount - 7.0f ));

		VVector sum;
		VVector pt;
		int n = points.size() - 1;
		int m = knotCount - 1;
		int p = m - n - 1;

		rasterized.clear();

	/*	glColor4f( 1.0f, 0.0f, 0.0f, 1.0f );
		glBegin( GL_LINE_STRIP );*/
		for ( float t = 0.0f; t <= 1.0; t += (float)1.0f/100.0f  )
		{
			sum = VVector( );
			for ( int i = 0; i <= n; i ++ )
			{
				pt = VVector( controlpoints[ 3*i ], controlpoints[ 3*i + 1 ], controlpoints[ 3*i + 2 ] );
				float coeff = getBasis( i, p, t, &knots );
				pt *= coeff;
				sum += pt;
				
			}
			
			//glVertex3fv( sum.getPtr() );
			rasterized.push_back( sum );

			if ( rasterized.size() > 1 )
			{
				VVector p1 = rasterized[ rasterized.size() - 2 ];
				VVector p2 = rasterized[ rasterized.size() - 1 ];
				float _line[ 4 ];
				_line[ 0 ] = p1.getX();
				_line[ 1 ] = p1.getY();
				_line[ 2 ] = p2.getX();
				_line[ 3 ] = p2.getY();

				if ( segmentIntersection( line , _line ) ) 
				{
					return true;
				}
			}
			
		}
		//glEnd();

		
	}
	
	return false;

}

bool Spline::hasIntersection( float *line )
{
	glLineWidth( 2.0f );

	//straighten control points
	std::vector<GLfloat> controlpoints;
	controlpoints.clear();
	for (unsigned int i =0; i< points.size() ;i++)
	{
		controlpoints.push_back((GLfloat)m_fBeta*points[i]->cart_x + (1-m_fBeta)*(points[0]->cart_x + i/(points.size()-1)*(points[points.size()-1]->cart_x - points[0]->cart_x)));
		controlpoints.push_back((GLfloat)m_fBeta*points[i]->cart_y + (1-m_fBeta)*(points[0]->cart_y + i/(points.size()-1)*(points[points.size()-1]->cart_y - points[0]->cart_y)));
		controlpoints.push_back((GLfloat)0);
	}

	bool intersects = false;

	// draw curve
	// with less points, the degree is smaller...
	if (points.size() < 3)
	{
		float knots[4] = { 0.0, 0.0, 1.0, 1.0 };

		VVector p1 = VVector( points[ 0 ]->cart_x , points[ 0 ]->cart_y , 0.0f );
		VVector p2 = VVector( points[ 1 ]->cart_x , points[ 1 ]->cart_y , 0.0f );
		rasterized.push_back( p1 );
		rasterized.push_back( p2 );	
		
		float _line[ 4 ];
		_line[ 0 ] = p1.getX();
		_line[ 1 ] = p1.getY();
		_line[ 2 ] = p2.getX();
		_line[ 3 ] = p2.getY();

		if ( segmentIntersection( line , _line ) ) 
		{
			return true;
		}
		
		
	}
	else if (points.size()==3)
	{
		
		VVector sum;
		VVector p;
		int n = points.size() - 1;
		rasterized.clear();

		/*glColor4f( 1.0f, 0.0f, 0.0f, 1.0f );
		glBegin( GL_LINE_STRIP );*/
		for ( float t = 0.0f; t <= 1.0; t += (float)0.01f  )
		{
			sum = VVector( );

			if ( t == 0.0 )
			{
				sum = VVector( controlpoints[ 0 ], controlpoints[ 1 ], controlpoints[ 2 ] );
			}
			else if ( t == 1.0 )
			{
				sum = VVector( controlpoints[ 3*n ], controlpoints[ 3*n + 1 ], controlpoints[ 3*n + 2 ] );
			}
			else 
			{

				for ( int i = 0; i <= n; i ++ )
				{
					p = VVector( controlpoints[ 3*i ], controlpoints[ 3*i + 1 ], controlpoints[ 3*i + 2 ] );
			
					double binome = factorial( (double)n ) / ( factorial( (double)i ) * factorial( (double)( n -i ) ));
					double factor1 = exp( i * log( t ) );
					double factor2 = exp( ( n - i ) * log( 1.0 - t ) ); 
					float factor = (float) ( binome * factor1 * factor2 );
					p *= factor;
					sum += p;
					
				}
			}

			//glVertex3fv( sum.getPtr() );
			rasterized.push_back( sum );

			if ( rasterized.size() > 1 )
			{
				VVector p1 = rasterized[ rasterized.size() - 2 ];
				VVector p2 = rasterized[ rasterized.size() - 1 ];
				float _line[ 4 ];
				_line[ 0 ] = p1.getX();
				_line[ 1 ] = p1.getY();
				_line[ 2 ] = p2.getX();
				_line[ 3 ] = p2.getY();

				if ( segmentIntersection( line , _line ) ) 
				{
					return true;
				}
			}
			
		}
		//glEnd();
		
	}
	else
	{

		//glColor3fv( m_vColorStart->getPtr() );
		int order = 4; // curve degree +1
		// number of knot vector = number of control points + degree + 1
		int knotCount = points.size()+ order;

		// create knots
		unsigned int i=0;
		std::vector<float> knots;
		knots.clear();

		// fill open uniform knot vector
		knots.push_back((float)0);
		knots.push_back((float)0);
		knots.push_back((float)0);
		for (i=0; i < knotCount-7; i++)
		{
			knots.push_back((float)i / (float)(knotCount - 7.0f ));
		}
		knots.push_back((float)i / (float)( knotCount - 7.0f ));
		knots.push_back((float)i / (float)( knotCount - 7.0f ));
		knots.push_back((float)i / (float)( knotCount - 7.0f ));
		knots.push_back((float)i / (float)( knotCount - 7.0f ));

		VVector sum;
		VVector pt;
		int n = points.size() - 1;
		int m = knotCount - 1;
		int p = m - n - 1;

		rasterized.clear();

		//glColor4f( 1.0f, 0.0f, 0.0f, 1.0f );
		//glBegin( GL_LINE_STRIP );
		for ( float t = 0.0f; t <= 1.0; t += (float)1.0f/100.0f  )
		{
			sum = VVector( );
			for ( int i = 0; i <= n; i ++ )
			{
				pt = VVector( controlpoints[ 3*i ], controlpoints[ 3*i + 1 ], controlpoints[ 3*i + 2 ] );
				float coeff = getBasis( i, p, t, &knots );
				pt *= coeff;
				sum += pt;
				
			}
			
			//glVertex3fv( sum.getPtr() );
			rasterized.push_back( sum );

			if ( rasterized.size() > 1 )
			{
				VVector p1 = rasterized[ rasterized.size() - 2 ];
				VVector p2 = rasterized[ rasterized.size() - 1 ];
				float _line[ 4 ];
				_line[ 0 ] = p1.getX();
				_line[ 1 ] = p1.getY();
				_line[ 2 ] = p2.getX();
				_line[ 3 ] = p2.getY();

				if ( segmentIntersection( line , _line ) ) 
				{
					return true;
				}
			}
			
		}
	//	glEnd();

		
	}
	
	return false;
}

float Spline::getBasis( int i, int p, float t, std::vector< float > *vknots )
{
	if( p == 0 )
	{
		if( (*vknots)[ i ] <= t && t < (*vknots)[ i + 1 ] && (*vknots)[ i ] < (*vknots)[ i + 1 ] )
			return 1.0f;
		else
			return 0.0f;
	}

	float b, b1, b2, factor1, factor2;
	float nom1, nom2, den1, den2;

	nom1 = t - (*vknots)[ i ];
	den1 = (*vknots)[ i + p ] - (*vknots)[ i ];
	nom2 = (*vknots)[ i + p + 1 ] - t;
	den2 = (*vknots)[ i + p + 1 ] - (*vknots)[ i + 1 ];

	b1 = getBasis( i, p - 1, t, vknots );
	b2 = getBasis( i + 1, p - 1, t, vknots );

	if( b1 != 0.0 )
	{
		if ( nom1 == 0.0f && den1 == 0.0f )
		{
			factor1 = 0.0f;
		}
		else
		{
			if ( den1 == 0.0f )
			{
				std::cout << "WARNING: division by 0!" << std::endl;
				factor1 = 0.0f;
			}
			else
			{
				factor1 = nom1 / den1;
			}
		}

		b1 *= factor1;
	}

	if( b2 != 0.0 )
	{
		if ( nom2 == 0.0f && den2 == 0.0f )
		{
			factor2 = 0.0f;
		}
		else
		{
			if ( den2 == 0.0f )
			{
				std::cout << "WARNING: division by 0!" << std::endl;
				factor2 = 0.0f;
			}
			else
			{
				factor2 = nom2 / den2;
			}
		}

		b2 *= factor2;
	}

	b = b1 + b2;
	

	return b;
}