#include "TreeRep.h"

#include "glew.h"
#include <gl/gl.h>			
#include <gl/glu.h>	

#include <iostream>
#include "Spline.h"

#define PI 3.14159265358979323846
#define EPSILON 0.01
float gscale  = 1.0f;

void TreeRep::setupTextures()
{
	tree< TreeEntity >::pre_order_iterator pi = m_TreeDatastructure.begin();

	while( pi != m_TreeDatastructure.end() )
	{
		if(  pi->m_ImagePresent && !pi->m_ImageCreated )
		{
			pi->createImage();
			pi->m_Image->loadImage( pi->m_ImagePath );
			pi->m_ImageCreated = true;
		}
		pi++;
	}
}

tree<TreeEntity>::iterator TreeRep::find( float x, float y, bool *success )
{
	tree< TreeEntity >::pre_order_iterator pi = m_TreeDatastructure.begin();

	while( pi != m_TreeDatastructure.end() )
	{
		float dx = pi->cart_x - x;
		float dy = pi->cart_y - y;

		if ( fabs( dx ) < /*gscale * */EPSILON && fabs( dy ) < /*gscale * */EPSILON )
		{
			(*success) = true;
			return (pi.node);
		}
		pi++;
	}
	(*success) = false;
	return pi;
}


void TreeRep::setScale( float factor )
{
	pointsize = factor * m_InitPointsize;
	pointsize =  ( pointsize < 1.0f ) ? 1.0f : pointsize;

	gscale = factor;
}

void TreeRep::clear()
{
	splines.clear();
}

void TreeRep::setEmphasize( float emph )
{
	m_fEmphasize = emph;

	for (int i = 0; i < splines.size(); i++ )
	{
		splines[ i ]->setEmphasize( emph );
	}
}

void TreeRep::setBeta( float beta )
{
	m_fBeta = beta;

	for (int i = 0; i < splines.size(); i++ )
	{
		splines[ i ]->setBeta( beta );
	}
}

void TreeRep::setAlpha( float alpha )
{
	m_fAlpha = alpha;

	for (int i = 0; i < splines.size(); i++ )
	{
		splines[ i ]->setAlpha( alpha );
	}
}

void TreeRep::createSplineRepresentation( std::vector< strRelationshipPair > *relations )
{
	int max_pt_number = 2;

	for( int i = 0; i < relations->size(); i++ )
	{
		strRelationshipPair rpair = (*relations)[ i ];

		//we won't create a spline between nodes a and b if b==a
		if ( rpair.id1 ==  rpair.id2 ) continue;

		bool swap = ( rpair.id1 > rpair.id2 );

		if ( swap )
		{
			//rpair.id1 += rpair.id2; // a := a+b
			//rpair.id2 = rpair.id1 - rpair.id2; //b := a+b - b = a
			//rpair.id1 -= rpair.id2; //a:= a+b - a = b
			int help = rpair.id1;
			rpair.id1 = rpair.id2;
			rpair.id2 = help;
		}

		assert( rpair.id1 <= rpair.id2 );

		//search for the first id (preorder)
		// - remember all traversed entities after the first travesed id is greater than the second id

		tree< TreeEntity >::iterator current = m_TopNode;
		tree< TreeEntity >::iterator lastcommon;
		std::vector< TreeEntity* > entities;

		bool starttracking = false;
		bool found1 = false;
		bool found2 = false;

		while( !found1 && current->numchildren > 0 )
		{
			int currentid = current->id;

			tree< TreeEntity >::iterator csibl = current.node->first_child;

			int num_siblings = current.number_of_children();

			for( int i = 0; i < num_siblings && !found1; i++ )
			{
				// - travese until first id is found
				if ( csibl->id >= rpair.id1 )
				{

					if ( csibl->id < rpair.id2 && !starttracking )
					{
						starttracking = true;
						lastcommon = current;
						//entities.push_back( &( current.node->data) );
						//saved the parent which is the deepest common node of the spline in hierarchy
						//from now on, saved every node traversed
					}

					if ( starttracking )
					{
						entities.push_back( &(csibl.node->data) );

					}

					if ( csibl->id == rpair.id1 )
					{
						found1 = true;
						break;
					}
					current = csibl;
					break;

				}

				csibl = csibl.node->next_sibling;
			}

		}
		
		// - reverse order in the vector of remembered entities so that the first id is first in the vector
		std::vector< TreeEntity* > tmp = entities;
		entities.clear();

		for ( int j = 1; j <= tmp.size(); j++ )
		{
			entities.push_back( tmp[ tmp.size() - j ] );
		}
		tmp.clear();

		// - continue search from the last entity in the vector until the second id is reached
		current = lastcommon;
		
		if ( current->id == rpair.id2 )
		{
			found2 = true;
			entities.push_back( &( current.node->data ) );
		}

		while( !found2 && current->numchildren > 0 )
		{
			int currentid = current->id;

			tree< TreeEntity >::iterator csibl = current.node->first_child;

			int num_siblings = current.number_of_children();

			for( int i = 0; i < num_siblings && !found2; i++ )
			{
				
				// - travese until last id is found
				if ( csibl->id >= rpair.id2 )
				{

					entities.push_back( &(csibl.node->data) );

					if ( csibl->id == rpair.id2 )
					{
						found2 = true;
						break;
					}
					current = csibl;
					break;

				}

				csibl = csibl.node->next_sibling;
			}

		}

		// if the spline would have only 2 points, we add the LCA in the middle
		if ( entities.size() == 2 /*if the spline would have only 2 points*/
			 && lastcommon->id != rpair.id1 /*if the LCA is not the first in the vector*/
			 && lastcommon->id != rpair.id2 /*if the LCA is not the last in the vector*/
			)
		{
			TreeEntity *te = entities[ 1 ];
			entities.pop_back();
			entities.push_back( &(lastcommon.node->data) );
			entities.push_back( te );
		}

		// - create a spline

		// alternative coloring
		////women are 1, men are 0 (sex)
		//VVector *c1 = ( entities[ 0 ]->sex ) ? m_vSplineColor_2 : m_vSplineColor_1;
		//VVector *c2 = ( entities[ entities.size() - 1 ]->sex ) ? m_vSplineColor_2 : m_vSplineColor_1;

		VVector *c1;
		VVector *c2;
		// source and dest have different colors
		if (swap)
		{
			c1 =  m_vSplineColor_2;
			c2 =  m_vSplineColor_1;
		}
		else
		{
			c1 =  m_vSplineColor_1;
			c2 =  m_vSplineColor_2;
		}

		Spline *s = new Spline( entities, c1, c2, m_fAlpha, m_fBeta );
		splines.push_back( s );

		if ( entities.size() > max_pt_number )
		{
			max_pt_number = entities.size();
		}
	}
	
	for ( int i = 0; i < splines.size(); i++ )
	{
		splines[ i ]->setLongest( max_pt_number );
	}

}

void TreeRep::hasIntersections( bool selective_a, float *line, float zoom, VVector *pan )
{

	if ( !m_bRelationsPresent ) return;
	if ( splines.empty() ) return;

	float transfline[ 4 ];
	transfline[ 0 ] = line[ 0 ]/ zoom - pan->getX();
	transfline[ 1 ] = line[ 1 ]/ zoom - pan->getY();
	transfline[ 2 ] = line[ 2 ]/ zoom - pan->getX();
	transfline[ 3 ] = line[ 3 ]/ zoom - pan->getY();
	
	if ( !selective_a ) 
	{
		for ( int i = 0; i < splines.size(); i++ )
		{
			splines[ i ]->setStatus( true );
		}
	}
	else
	{
		for ( int i = 0; i < splines.size(); i++ )
		{
			bool inters = splines[ i ]->hasIntersection( transfline );
			splines[ i ]->setStatus( inters );
		}
	}
}

void TreeRep::hasIntersectionsSpecial( bool selective_a, float *line, float zoom, VVector *pan )
{

	if ( !m_bRelationsPresent ) return;
	if ( splines.empty() ) return;

	float transfline[ 4 ];
	transfline[ 0 ] = line[ 0 ]/ zoom - pan->getX();
	transfline[ 1 ] = line[ 1 ]/ zoom - pan->getY();
	transfline[ 2 ] = line[ 2 ]/ zoom - pan->getX();
	transfline[ 3 ] = line[ 3 ]/ zoom - pan->getY();
	
	if ( !selective_a ) 
	{
		for ( int i = 0; i < splines.size(); i++ )
		{
			splines[ i ]->setStatus( true );
		}
	}
	else
	{
		for ( int i = 0; i < splines.size(); i++ )
		{
			bool inters = splines[ i ]->hasIntersectionSpecial( transfline );
			splines[ i ]->setStatus( inters );
		}
	}
}

void TreeRep::drawSplinesNormal(  )
{
	if ( !m_bRelationsPresent ) return;
	if ( splines.empty() ) return;

	for ( int i = 0; i < splines.size(); i++ )
	{
		splines[ i ]->draw();
		//splines[ i ]->hasIntersection(0.0f,0.0f,0.0f,0.0f);//draw();
	}
}

void TreeRep::drawSplinesSpecial(  )
{
	if ( !m_bRelationsPresent ) return;
	if ( splines.empty() ) return;

	for ( int i = 0; i < splines.size(); i++ )
	{
		splines[ i ]->drawInvRooted();
	}
}

void TreeRep::drawRadial()
{
	//pos x = radius
	//pos y = phi

	if ( !m_bDataPresent ) return;
	
	m_TopNode.node->data.pos_x = 0.0f;
	m_TopNode.node->data.pos_y = 0.0f;
	m_TopNode.node->data.extent = 2.0 * PI;
	m_TopNode.node->data.numchildren = m_TopNode.number_of_children();

	m_TopNode.node->data.cart_x = m_TopNode.node->data.pos_x;
	m_TopNode.node->data.cart_y = m_TopNode.node->data.pos_y;

	glPointSize( 2.0f * pointsize );
	glLineWidth( linewidth );

	glHint( GL_POINT_SMOOTH_HINT, GL_NICEST );
	glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );

	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);

	glDepthFunc( GL_ALWAYS );
	glColor3fv( m_vPointColor->getPtr() );
	glBegin( GL_POINTS );
		glVertex2f( 0.0f, 0.0f );
	glEnd();

	glPointSize( pointsize );
	
	float depth = 0;

	if(m_TopNode != m_TreeDatastructure.end()) 
	{
		tree<TreeEntity>::fixed_depth_iterator fdi;

		tree<TreeEntity>::iterator _firstNodeNextDepth = m_TopNode;
		tree<TreeEntity>::iterator _currentParentNode = m_TopNode;

		while ( _firstNodeNextDepth.number_of_children() > 0)
		{
			fdi = m_TreeDatastructure.begin( _currentParentNode );

			bool nextset = false;
			int order = 0;

			depth++;

			while(fdi!= m_TreeDatastructure.end( _currentParentNode ) ) 
			{

				if( !nextset )
				{
					_firstNodeNextDepth = fdi;
					nextset = true;
				}else

				if( _firstNodeNextDepth.number_of_children() == 0 )
				{
					_firstNodeNextDepth = fdi;
				}

				(*fdi).numchildren = fdi.number_of_children();
				(*fdi).order = order;
				
				//compute the position of the node, pos x = radius
				(*fdi).pos_x = depth / (float)( m_iNumHierarchies + 2 );

				if ( depth < 2.0f )
				{
					
					(*fdi).pos_y =    fdi.node->parent->data.pos_y 
									- 0.5f * fdi.node->parent->data.extent
									+ (float)order * fdi.node->parent->data.extent / ( (float)fdi.node->parent->data.numchildren );
					(*fdi).extent = (PI) / ( (float)fdi.node->parent->data.numchildren + 1.0f );
				
				}else
				{
					(*fdi).pos_y =    fdi.node->parent->data.pos_y 
									- 0.5f * fdi.node->parent->data.extent
									+ (float)order * fdi.node->parent->data.extent / ( (float)fdi.node->parent->data.numchildren - 1.0f );
					(*fdi).extent = fdi.node->parent->data.extent / ( (float)fdi.node->parent->data.numchildren + 1.0f );
				}
				

				float radius = (*fdi).pos_x;
				float phi = (*fdi).pos_y;
				float extent = (*fdi).extent;

				float x = fdi.node->data.pos_x * cos ( fdi.node->data.pos_y );
				float y = fdi.node->data.pos_x * sin ( fdi.node->data.pos_y );

				float px = fdi.node->parent->data.pos_x * cos ( fdi.node->parent->data.pos_y );
				float py = fdi.node->parent->data.pos_x * sin ( fdi.node->parent->data.pos_y );

				glDepthFunc( GL_ALWAYS );
				glColor3fv( m_vPointColor->getPtr() );
				glBegin( GL_POINTS );
					glVertex2f( x, y );
				glEnd();

				glDepthFunc( GL_GREATER );
				glColor3fv( m_vLineColor->getPtr() );
				glBegin( GL_LINES );
					glVertex3f( x, y, -0.1f );
					glVertex3f( px, py, -0.1f );
				glEnd();

				(*fdi).cart_x = x;
				(*fdi).cart_y = y;

				order = ( fdi.node->parent->data.numchildren - 1 == order ) ? 0 : order + 1;

				fdi++;
			}
			
			
			_currentParentNode = _firstNodeNextDepth;
			nextset = false;
		}

	}
}

void TreeRep::drawInvRadial()
{
	if ( !m_bDataPresent ) return;

	//pos x radius
	//pos y mitte winkel
	//extent = extent winkel

	m_TopNode.node->data.pos_x = PI / 2.0f;
	m_TopNode.node->data.pos_y = 0.0f;
	m_TopNode.node->data.extent = 2.0 * PI;
	m_TopNode.node->data.numchildren = m_TopNode.number_of_children();
	m_TopNode.node->data.cart_x = 0.0f;
	m_TopNode.node->data.cart_y = 0.0f;

	glPointSize( 2.0f * pointsize );
	glLineWidth( linewidth );
	glPointSize( pointsize );

	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);

	glHint( GL_POINT_SMOOTH_HINT, GL_NICEST );
	glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );
	
	
	/*glBegin( GL_POINTS );
	glVertex2f( m_TopNode.node->data.cart_x, m_TopNode.node->data.cart_y );
	glEnd();*/

	float depth = 0;

	const float phi_skip = 0.1f;
	float circle_thickness = 0.1f;
	float max_thickness = min( 1.0f, circle_thickness + (float)m_iNumHierarchies / ( m_iNumHierarchies + 2.0f ) );
	circle_thickness = max_thickness - (float)m_iNumHierarchies / ( m_iNumHierarchies + 2.0f );

	if(m_TopNode != m_TreeDatastructure.end()) 
	{
		tree<TreeEntity>::fixed_depth_iterator fdi;

		tree<TreeEntity>::iterator _firstNodeNextDepth = m_TopNode;
		tree<TreeEntity>::iterator _currentParentNode = m_TopNode;
		
		glDepthFunc( GL_GREATER );
		glColor3fv( m_vLineColor->getPtr() );
		glBegin( GL_LINE_LOOP );

		for ( float k = 0.0f; k < 100.0f; k++ )
		{
			float phi = ( k / 100.0f )* 2.0f * PI;

			glVertex3f( max_thickness * cos ( phi ), 
						max_thickness * sin ( phi ),
						-0.1f);
		}

		glEnd( );

		while ( _firstNodeNextDepth.number_of_children() > 0)
		{
			fdi = m_TreeDatastructure.begin( _currentParentNode );

			bool nextset = false;
			int order = 0;

			depth++;

			while(fdi!= m_TreeDatastructure.end( _currentParentNode ) ) 
			{

				if( !nextset )
				{
					_firstNodeNextDepth = fdi;
					nextset = true;
				}else

				if( _firstNodeNextDepth.number_of_children() == 0 )
				{
					_firstNodeNextDepth = fdi;
				}

				(*fdi).numchildren = fdi.number_of_children();
				(*fdi).order = order;

				glPointSize( pointsize );
				glLineWidth( linewidth );

				glHint( GL_POINT_SMOOTH_HINT, GL_NICEST );
				glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );

				glEnable(GL_LINE_SMOOTH);
				glEnable(GL_POINT_SMOOTH);
				
				if ( (*fdi).numchildren > 0 && depth < 2 )
				{

					//compute the position of the node, pos x = radius
					(*fdi).pos_x = fdi.node->parent->data.pos_x 
						+ order * ( fdi.node->parent->data.extent - ( fdi.node->parent->data.numchildren ) * phi_skip ) / fdi.node->parent->data.numchildren 
						+ ( order ) * phi_skip;
					(*fdi).pos_y = 0.0f;

					(*fdi).extent = ( fdi.node->parent->data.extent - ( fdi.node->parent->data.numchildren ) * phi_skip ) / fdi.node->parent->data.numchildren;
					
					float posx = (*fdi).pos_x;
					float extent = (*fdi).extent;

					int steps = ( 100.0f * (*fdi).extent / ( 2 * PI ) + 0.5f );

					glDepthFunc( GL_GREATER );
					glColor3fv( m_vLineColor->getPtr() );
					glBegin( GL_LINE_STRIP );

					for ( float k = 0.0f; k < steps; k++ )
					{
						float phi = (*fdi).pos_x + ( k / 100.0f )* 2.0f * PI;

						glVertex3f( ( max_thickness - ( circle_thickness ) * depth / ( (float)m_iNumHierarchies  ) ) * cos ( phi ), 
									( max_thickness - ( circle_thickness ) * depth / ( (float)m_iNumHierarchies  ) ) * sin ( phi ),
									  -0.1f);
					}

					float phi = (*fdi).pos_x + (*fdi).extent;

					glVertex3f( ( max_thickness - ( circle_thickness ) * depth / ( (float)m_iNumHierarchies  ) ) * cos ( phi ), 
								( max_thickness - ( circle_thickness ) * depth / ( (float)m_iNumHierarchies  ) ) * sin ( phi ),
								-0.1f);


					glEnd( );
					
					(*fdi).cart_x = ( depth / (float)( m_iNumHierarchies + 2 ) ) * cos ( (*fdi).pos_x + (*fdi).extent / 2.0f );
					(*fdi).cart_y = ( depth / (float)( m_iNumHierarchies + 2 ) ) * sin ( (*fdi).pos_x + (*fdi).extent / 2.0f );

					/*glBegin( GL_POINTS );
					glVertex2f( (*fdi).cart_x, (*fdi).cart_y );
					glEnd();*/

					order = ( fdi.node->parent->data.numchildren - 1 == order ) ? 0 : order + 1;

					
				}
				else if ( (*fdi).numchildren > 0 )
				{
					//compute the position of the node, pos x = radius
					(*fdi).pos_x = fdi.node->parent->data.pos_x 
						/*number of previous children*/
						+ order * ( fdi.node->parent->data.extent - ( fdi.node->parent->data.numchildren -1 ) * phi_skip ) / ( fdi.node->parent->data.numchildren ) 
						+ order * phi_skip;
					(*fdi).pos_y = 0.0f;

					(*fdi).extent = ( fdi.node->parent->data.extent - ( fdi.node->parent->data.numchildren -1 ) * phi_skip ) / fdi.node->parent->data.numchildren;
					
					int steps = ( 100.0f * (*fdi).extent / ( 2 * PI ) + 0.5f );

					glDepthFunc( GL_GREATER );
					glColor3fv( m_vLineColor->getPtr() );
					glBegin( GL_LINE_STRIP );

					for ( float k = 0.0f; k < steps; k++ )
					{
						float phi = (*fdi).pos_x + ( k / 100.0f )* 2.0f * PI;

						glVertex3f( ( max_thickness - ( circle_thickness ) * depth / ( (float)m_iNumHierarchies ) ) * cos ( phi ), 
									( max_thickness - ( circle_thickness ) * depth / ( (float)m_iNumHierarchies ) ) * sin ( phi ),
									-0.1f);
					}

					float phi = (*fdi).pos_x + (*fdi).extent;

					glVertex3f( ( max_thickness - ( circle_thickness ) * depth / ( (float)m_iNumHierarchies  ) ) * cos ( phi ), 
								( max_thickness - ( circle_thickness ) * depth / ( (float)m_iNumHierarchies  ) ) * sin ( phi ),
								-0.1f);

					glEnd( );

					(*fdi).cart_x = ( depth / (float)( m_iNumHierarchies + 2 ) ) * cos ( (*fdi).pos_x + (*fdi).extent / 2.0f );
					(*fdi).cart_y = ( depth / (float)( m_iNumHierarchies + 2 ) ) * sin ( (*fdi).pos_x + (*fdi).extent / 2.0f );

				/*	glBegin( GL_POINTS );
					glVertex2f( (*fdi).cart_x, (*fdi).cart_y );
					glEnd();*/

					order = ( fdi.node->parent->data.numchildren - 1 == order ) ? 0 : order + 1;

				}
				else
				{
					(*fdi).pos_x = fdi.node->parent->data.pos_x 
						/*number of previous children*/
						/*0.5 because of the 0.5 shift in the circle*/
						+ ( order + 1.0f - 0.5f ) * ( fdi.node->parent->data.extent / fdi.node->parent->data.numchildren );
					(*fdi).pos_y = 0.0f;
					order = ( fdi.node->parent->data.numchildren - 1 == order ) ? 0 : order + 1;

					const float phi = (*fdi).pos_x;
					
					glDepthFunc( GL_GREATER );
					glColor3fv( m_vLineColor->getPtr() );
					glBegin( GL_LINES );
						glVertex3f( ( depth / (float)( m_iNumHierarchies + 2 ) ) * cos ( phi ), 
									( depth / (float)( m_iNumHierarchies + 2 ) ) * sin ( phi ),
									-0.1f);
						glVertex3f( ( max_thickness - ( circle_thickness ) * ( - 1.0f + depth )      / ( (float)m_iNumHierarchies  ) ) * cos ( phi ), 
									( max_thickness - ( circle_thickness ) * ( - 1.0f + depth )      / ( (float)m_iNumHierarchies  ) ) * sin ( phi ),
									-0.1f);
					glEnd();

					(*fdi).cart_x = ( depth / (float)( m_iNumHierarchies + 2 ) ) * cos ( phi );
					(*fdi).cart_y = ( depth / (float)( m_iNumHierarchies + 2 ) ) * sin ( phi );

					glDepthFunc( GL_ALWAYS );
					glColor3fv( m_vPointColor->getPtr() );
					glBegin( GL_POINTS );
					glVertex2f( (*fdi).cart_x, (*fdi).cart_y );
					glEnd();
				
				}

				fdi++;

			}

			_currentParentNode = _firstNodeNextDepth;
			nextset = false;
		}

	}


}

void TreeRep::drawRooted(  )
{
	if ( !m_bDataPresent ) return;
	
	m_TopNode.node->data.pos_x = 0.0f;
	m_TopNode.node->data.pos_y = 1.0f - 1.0f / (float)( m_iNumHierarchies + 1.0f );
	m_TopNode.node->data.extent = 2.0f;
	m_TopNode.node->data.numchildren = m_TopNode.number_of_children();

	m_TopNode.node->data.cart_x = m_TopNode.node->data.pos_x;
	m_TopNode.node->data.cart_y = m_TopNode.node->data.pos_y;

	
	glLineWidth( linewidth );

	glEnable(GL_LINE_SMOOTH);
	glEnable(GL_POINT_SMOOTH);

	glHint( GL_POINT_SMOOTH_HINT, GL_NICEST );
	glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );

	glPointSize( 2.0f * pointsize );
	glDepthFunc( GL_ALWAYS );
	glColor3fv( m_vPointColor->getPtr() );
	glBegin( GL_POINTS );
		glVertex2f( m_TopNode.node->data.pos_x, m_TopNode.node->data.pos_y );
	glEnd();

	glPointSize( pointsize );

	if(m_TopNode != m_TreeDatastructure.end()) 
	{
		tree<TreeEntity>::fixed_depth_iterator fdi;

		tree<TreeEntity>::iterator _firstNodeNextDepth = m_TopNode;
		tree<TreeEntity>::iterator _currentParentNode = m_TopNode;

		while ( _firstNodeNextDepth.number_of_children() > 0)
		{
			fdi = m_TreeDatastructure.begin( _currentParentNode );

			bool nextset = false;
			int order = 1;

			while(fdi!= m_TreeDatastructure.end( _currentParentNode ) ) 
			{
				if( !nextset )
				{
					_firstNodeNextDepth = fdi;
					nextset = true;
				}else

				if( _firstNodeNextDepth.number_of_children() == 0 )
				{
					_firstNodeNextDepth = fdi;
				}

				(*fdi).numchildren = fdi.number_of_children();
				(*fdi).order = order;
				
				//compute the position of the node
				(*fdi).pos_x =	  fdi.node->parent->data.pos_x 
								- fdi.node->parent->data.extent / 2.0f
								+ order * ( fdi.node->parent->data.extent )/( fdi.node->parent->data.numchildren + 1.0f );

				(*fdi).pos_y = fdi.node->parent->data.pos_y - 2.0f / (float)( m_iNumHierarchies + 1.0f );

				float pos_x = (*fdi).pos_x;
				float pos_y = (*fdi).pos_y;
				float extent = (*fdi).extent;

				(*fdi).extent = fdi.node->parent->data.extent / ( (float)fdi.node->parent->data.numchildren + 1.0f );

				glDepthFunc( GL_ALWAYS );
				glColor3fv( m_vPointColor->getPtr() );
				glBegin( GL_POINTS );
					glVertex2f( fdi.node->data.pos_x, fdi.node->data.pos_y );
				glEnd();

				(*fdi).cart_x = fdi.node->data.pos_x;
				(*fdi).cart_y = fdi.node->data.pos_y;

				glDepthFunc( GL_GREATER );
				glColor3fv( m_vLineColor->getPtr() );
				glBegin( GL_LINES );
					glVertex3f( fdi.node->data.pos_x, fdi.node->data.pos_y,-0.1f );
					glVertex3f( fdi.node->parent->data.pos_x, fdi.node->parent->data.pos_y,-0.1f );
				glEnd();

				order = ( fdi.node->parent->data.numchildren == order ) ? 1 : order + 1;

				fdi++;
			}

			_currentParentNode = _firstNodeNextDepth;
			nextset = false;
		}
      
   }
	
}


void TreeRep::drawInvRooted()
{
	if ( !m_bDataPresent ) return;
		
		m_TopNode.node->data.pos_x = 0.0f;
		m_TopNode.node->data.pos_y = 1.0f - 1.0f / (float)( 2.0f* m_iNumHierarchies + 0.5f );
		m_TopNode.node->data.extent = 2.0f;
		m_TopNode.node->data.numchildren = m_TopNode.number_of_children();

		m_TopNode.node->data.cart_x =   m_TopNode.node->data.pos_x;
		m_TopNode.node->data.cart_y = - m_TopNode.node->data.pos_y;

		glLineWidth( linewidth );

		glHint( GL_POINT_SMOOTH_HINT, GL_NICEST );
		glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );

		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_POINT_SMOOTH);

		glPointSize( 2.0f * pointsize );

		glDepthFunc( GL_ALWAYS );
		glColor3fv( m_vPointColor->getPtr() );
		glBegin( GL_POINTS );
			glVertex2f( m_TopNode.node->data.pos_x, m_TopNode.node->data.pos_y );
		glEnd();

		glPointSize( pointsize );

		if(m_TopNode != m_TreeDatastructure.end()) 
		{
			tree<TreeEntity>::fixed_depth_iterator fdi;

			tree<TreeEntity>::iterator _firstNodeNextDepth = m_TopNode;
			tree<TreeEntity>::iterator _currentParentNode = m_TopNode;

			while ( _firstNodeNextDepth.number_of_children() > 0)
			{
				fdi = m_TreeDatastructure.begin( _currentParentNode );

				bool nextset = false;
				int order = 1;

				while(fdi!= m_TreeDatastructure.end( _currentParentNode ) ) 
				{
					if( !nextset )
					{
						_firstNodeNextDepth = fdi;
						nextset = true;
					}else

					if( _firstNodeNextDepth.number_of_children() == 0 )
					{
						_firstNodeNextDepth = fdi;
					}

					(*fdi).numchildren = fdi.number_of_children();
					(*fdi).order = order;
					
					//compute the position of the node
					(*fdi).pos_x =	  fdi.node->parent->data.pos_x 
									- fdi.node->parent->data.extent / 2.0f
									+ order * ( fdi.node->parent->data.extent )/( fdi.node->parent->data.numchildren + 1.0f );

					(*fdi).pos_y = fdi.node->parent->data.pos_y - 2.0f / (float)( 2.0f * m_iNumHierarchies + 0.5f );
					
					(*fdi).extent = fdi.node->parent->data.extent / ( (float)fdi.node->parent->data.numchildren + 1.0f );

					glDepthFunc( GL_ALWAYS );
					glColor3fv( m_vPointColor->getPtr() );
					glBegin( GL_POINTS );
						glVertex2f( fdi.node->data.pos_x, fdi.node->data.pos_y );
					glEnd();

					(*fdi).cart_x =   fdi.node->data.pos_x;
					(*fdi).cart_y =   fdi.node->data.pos_y;

					glDepthFunc( GL_GREATER );
					glColor3fv( m_vLineColor->getPtr() );
					glBegin( GL_LINES );
						glVertex3f( fdi.node->data.pos_x, fdi.node->data.pos_y,-0.1f );
						glVertex3f( fdi.node->parent->data.pos_x, fdi.node->parent->data.pos_y,-0.1f );
					glEnd();

					order = ( fdi.node->parent->data.numchildren == order ) ? 1 : order + 1;

					fdi++;
				}

				_currentParentNode = _firstNodeNextDepth;
				nextset = false;
			}
	      
	   }
	
}

void TreeRep::computeBalloon()
{
		if ( !m_bDataPresent ) return;


		if(m_TopNode != m_TreeDatastructure.end()) 
		{
			float estimateFactor = 1.2f; // default 1.2  estimation factor for radii
			double factor = 1.1;  // some estimation factor for radii 
			bool evenAngles = false;

			// for all nodes set child counter
			tree<TreeEntity>::iterator iter;
			iter = m_TreeDatastructure.begin();
			while (iter != m_TreeDatastructure.end())
			{
				// reset
				iter.node->data.angle = 0.0f;
				iter.node->data.innerRadius = 0.0f;
				iter.node->data.outerRadius = 0.0f;
				iter.node->data.angle = 0.0f;
				iter.node->data.size = 0.01f;
				iter.node->data.maxChildRadius = 0.0f;
				iter.node->data.estimate = 0.0f;

				// asigning numchildren
				iter.node->data.numchildren = iter.number_of_children();
				iter.node->data.childCount = iter.number_of_children();
				iter.node->data.size = pointsize/400;
				iter++;
			}

			//***********************************************
			// COMPUTE RADII
			//***********************************************
			// crete queue for nodes
			std::queue<tree<TreeEntity>::iterator> q;
			
			// set radii for all leaves
			tree<TreeEntity>::leaf_iterator leaf;
			leaf = m_TreeDatastructure.begin_leaf();
			while (leaf != m_TreeDatastructure.end_leaf())
			{
				// inner and outer radius for leaf nodes are the same
				leaf.node->data.outerRadius = leaf.node->data.size;
				leaf.node->data.innerRadius = leaf.node->data.outerRadius;

				tree<TreeEntity>::iterator _currentParentNode = m_TreeDatastructure.parent(leaf);

				if (m_TreeDatastructure.is_valid(_currentParentNode))
				{
					// sum up outer radii of the children
					// to create an estimator for the inner radius of the parent.
					double t = leaf.node->data.outerRadius;
					_currentParentNode.node->data.estimate +=2.0*t;

					if ( _currentParentNode.node->data.maxChildRadius < t)
						_currentParentNode.node->data.maxChildRadius = t;

					// if all children are processed, push parrent
					if (--_currentParentNode.node->data.childCount == 0)
					{
						q.push(_currentParentNode);
					}

				} // end _currentParrentNode
				leaf++;
			} // end while leaf
			
			tree<TreeEntity>::iterator _currentNode;

			while(!q.empty())
			{
				_currentNode = q.front();
				q.pop();

				// compute inner radius of current node
				if (evenAngles)
				{
					_currentNode.node->data.innerRadius = max((_currentNode.node->data.maxChildRadius/max(_currentNode.node->data.numchildren, 1) + 
										estimateFactor*2.0*(_currentNode.node->data.numchildren*_currentNode.node->data.maxChildRadius))/
										(2*PI), 2.0*_currentNode.node->data.size);
					std::cout<<"Even Angles!\n";
				}
				else {
					if ( _currentNode.node->data.numchildren ==1)
					{
						_currentNode.node->data.innerRadius = max( 2.0*_currentNode.node->data.size , factor*_currentNode.node->data.maxChildRadius);
					}
					else
					{
						//_currentNode.node->data.innerRadius = max(max((_currentNode.node->data.maxChildRadius/
						//										max(_currentNode.node->data.numchildren, 4) + estimateFactor*2.0*_currentNode.node->data.estimate)/
						//										(2.0*PI), 2*_currentNode.node->data.size), factor*_currentNode.node->data.maxChildRadius);
						_currentNode.node->data.innerRadius = _currentNode.node->data.maxChildRadius/max(_currentNode.node->data.numchildren, 4) +estimateFactor*2.0*_currentNode.node->data.estimate/(2.0*PI);
					}
				} //end if( evenAngles)

				// compute outer radius of current node
				double t;
				if(_currentNode.node->data.numchildren ==1)
				{
					t = max(_currentNode.node->data.innerRadius, _currentNode.node->data.maxChildRadius);
				}
				else 
				{
					t = _currentNode.node->data.innerRadius + _currentNode.node->data.maxChildRadius;
				}
				_currentNode.node->data.outerRadius = t;

				// adjust parent values
				tree<TreeEntity>::iterator _parent = m_TreeDatastructure.parent(_currentNode);
				if(m_TreeDatastructure.is_valid(_parent))
				{
					_parent.node->data.estimate +=t;

					if ( _parent.node->data.maxChildRadius < t)
						_parent.node->data.maxChildRadius = t;

					// if all children are processed, push parrent
					if (--_parent.node->data.childCount == 0)
					{
						q.push(_parent);
					}

				} // end if (is_valid(_parent))

			} // end while( !q.empty())

			//***********************************************
			// COMPUTE ANGLES
			//***********************************************

			// reuse queue!
			// make sure it's empty
			/*while (!q.empty())
			{ 
				q.pop(); 
					std::cout<<"Queue POPPED!";
			}*/
			q.push(m_TopNode);
			while (! q.empty())
			{
				_currentNode = q.front();
				q.pop();

				if (m_TreeDatastructure.number_of_children(_currentNode) > 0)
				{
					double anglesum = 0.0;
					double fullAngle = 2.0*PI;
					double nodeEstimate = _currentNode.node->data.estimate;

					tree<TreeEntity>::sibling_iterator child = _currentNode.node->first_child;
					if (m_TreeDatastructure.number_of_children(_currentNode)==1)
					{
						child.node->data.angle = PI;
						q.push(child);
					}
					else
					{
						//std::cout<<"check ratio...\n";
						if (!evenAngles)
						{
							tree<TreeEntity>::sibling_iterator child2 = child;
							while(m_TreeDatastructure.is_valid(child2))
							{
								if (child2.node->data.outerRadius/_currentNode.node->data.estimate >0.501)
								{
									nodeEstimate = nodeEstimate - child2.node->data.outerRadius;
									fullAngle = PI;
								}
								child2++;
							} // while child2
						}
						else
						{
							std::cout<<"Even Angles!\n";
						} // end if not even angles
						
						while(m_TreeDatastructure.is_valid(child))
						{
							if(evenAngles)
							{
								child.node->data.angle = PI*2.0/m_TreeDatastructure.number_of_children(_currentNode);
								q.push(child);
							}
							else 
							{
								q.push(child);

								double ratio = child.node->data.outerRadius/_currentNode.node->data.estimate;

								if(ratio > 0.501) anglesum = PI;
								else anglesum = fullAngle*child.node->data.outerRadius/nodeEstimate;

								child.node->data.angle = anglesum;
							} // end uneven angles
							child++;
						} // end while child

					} // end else if number of children >1

				} // end if number_of_children > 0
			} // while !q.empty()

			//***********************************************
			// COMPUTE COORDINATES
			//***********************************************

		m_TopNode.node->data.pos_x = 0.0f;
		m_TopNode.node->data.pos_y = 0.0f;
		m_TopNode.node->data.extent = 2.0f;

		m_TopNode.node->data.cart_x = m_TopNode.node->data.pos_x;
		m_TopNode.node->data.cart_y = m_TopNode.node->data.pos_y;


		q.push(m_TopNode);
		while (! q.empty())
		{
			_currentNode = q.front();
			q.pop();

			double x = _currentNode.node->data.pos_x;
			double y = _currentNode.node->data.pos_y;

			if (m_TreeDatastructure.number_of_children(_currentNode) > 0)
			{
				double anglesum = fmod(_currentNode.node->data.angle+PI+
					_currentNode.node->first_child->data.angle/2.0, 2.0*PI);
				//double sumchecker = 0.0; debug value

				tree<TreeEntity>::sibling_iterator child = _currentNode.node->first_child;

				if (m_TreeDatastructure.number_of_children(_currentNode)==1)
				{
					q.push(child);
					// if there is only one child, it has same direction as parent node
					anglesum = _currentNode.node->data.angle;
					child.node->data.angle = anglesum;
					child.node->data.pos_x = x+cos(anglesum)*_currentNode.node->data.innerRadius;
					child.node->data.pos_y = y+sin(anglesum)*_currentNode.node->data.innerRadius;
					// copy to cart_xy
					child.node->data.cart_x = child.node->data.pos_x;
					child.node->data.cart_y = child.node->data.pos_y;

				}
				else 
				{
					while (m_TreeDatastructure.is_valid(child))
					{
						q.push(child);
						tree<TreeEntity>::sibling_iterator sibling;

						child++;
						if (m_TreeDatastructure.is_valid(child))
							sibling = child;
						else sibling = ((_currentNode.node->first_child));
						child--;

						// compute coordinates for current child
						child.node->data.pos_x = x+cos(anglesum)*_currentNode.node->data.innerRadius;
						child.node->data.pos_y = y+sin(anglesum)*_currentNode.node->data.innerRadius;
						// copy to cart_xy
						child.node->data.cart_x = child.node->data.pos_x;
						child.node->data.cart_y = child.node->data.pos_y;


						//sumchecker +=currentChild.angle;

						child.node->data.angle = anglesum;
						anglesum = fmod((anglesum +(child.node->data.angle+sibling.node->data.angle)/2.0), 2.0*PI);
						child++;
					} // while child valid
				} // end else 
			} // end if children
		}// while queue
			std::cout<<"computeBalloon()\n";
	}
}

void TreeRep::drawBalloon()
{

	if ( !m_bDataPresent ) return;


	if(m_TopNode != m_TreeDatastructure.end()) 
	{


		//***********************************************
		// DRAW COORDINATES
		//***********************************************

		glPointSize( pointsize );
		glLineWidth( linewidth );

		glHint( GL_POINT_SMOOTH_HINT, GL_NICEST );
		glHint( GL_LINE_SMOOTH_HINT, GL_NICEST );

		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_POINT_SMOOTH);

		tree<TreeEntity>::iterator iter;

		iter = m_TreeDatastructure.begin();
		while (iter != m_TreeDatastructure.end())
		{
			double scaling = 1.0; // SCALING!
			
			
			if (m_TreeDatastructure.is_valid(iter.node->parent))
			{
	
				glDepthFunc( GL_GREATER );
				glColor3fv( m_vLineColor->getPtr() );
				glBegin( GL_LINES );
					glVertex3f( iter.node->data.cart_x*scaling, iter.node->data.cart_y*scaling, -0.1f );
					glVertex3f( iter.node->parent->data.cart_x*scaling, iter.node->parent->data.cart_y*scaling, -0.1f );
				glEnd();
			}

			if (m_TreeDatastructure.is_valid(iter.node->first_child))
			{
				
				glDepthFunc( GL_GREATER );
				glColor4f( m_vLineColor->getX(), m_vLineColor->getY(), m_vLineColor->getZ(), 0.5f );
				glLineWidth( 0.5f * linewidth );
				glBegin( GL_LINE_LOOP );
					double pos_x = iter.node->data.cart_x*scaling;
					double pos_y = iter.node->data.cart_y*scaling;
					double radius = iter.node->data.innerRadius*scaling;
					for( int k = 0; k < 30; k++ ) {
						float theta = 2.0 * 3.14 / 30 * k;
						glVertex3d( pos_x + radius * sin( theta ), 
							pos_y + radius * cos( theta ), -0.2 );
					}
				glEnd();
				glLineWidth( linewidth );
				
				
			}

			glDepthFunc( GL_ALWAYS );
			glColor3fv( m_vPointColor->getPtr() );
			glBegin( GL_POINTS );
			glVertex2f( iter.node->data.cart_x*scaling, iter.node->data.cart_y*scaling );
			glEnd();

			iter++;
		}

		glDepthFunc( GL_ALWAYS );
		glColor3fv( m_vPointColor->getPtr() );
		glPointSize( 2.0f * pointsize );

		glBegin( GL_POINTS );
		glVertex2f( m_TopNode.node->data.cart_x, m_TopNode.node->data.cart_y );
		glEnd();

		glPointSize( pointsize );
		//std::cout<<"Balloon Layout finished\n";
		
	   } //end if(m_TopNode != m_TreeDatastructure.end())
	
}
