#ifndef _TREE_REP_
#define _TREE_REP_

#include "tree.h"
#include "TreeEntity.h"
#include "VVector.h"
#include "Spline.h"

#include <vector>


/**
 * struct encapsulating two integers representing the ids
 */
struct strRelationshipPair
{
	int id1, id2;

	strRelationshipPair( int _id1, int _id2 )
	{
		id1 = _id1;
		id2 = _id2;
	}

};

class TreeRep
{
public: 

	/**
	 * default constructor
	 */
	TreeRep()
	{
		m_bDataPresent = false;
		m_bRelationsPresent = false;

		pointsize = 10.0f;
		linewidth = 2.0f;

		m_InitPointsize = 10.0f;
		m_InitLinewidth = 2.0f;

		m_fAlpha= 0.5f;
		m_fBeta = 0.5f;

	}

	/**
	 * default destructor
	 */
	~TreeRep()
	{
		splines.clear();
	
	}

	/**
	* finds the entity in the tree dtastructure which lies under the click
	* @param x is the x coordinate
	* @param y is the y coordinate
	* @param success is a pointer to write whearther th eop was successful
	* @return the entity as iterator which underlies the coordinate pair (x,y)
	*/
	tree<TreeEntity>::iterator find( float x, float y, bool *success );

	void setupTextures();

	/**
	* clears all splines slines
	*/
	void clear(  );

	/**
	* sets the scale to adjust the pointsize where minimal pointsize is 1.0f
	* @param factor is the scaling factor
	*/
	void setScale( float factor );

	/**
	* sets the alpha parameter to all slines
	* @param apha is the new value of the alpha parameter
	*/
	void setAlpha( float alpha );

	/**
	* sets the beta parameter to all slines
	* @param beta is the new value of the beta parameter
	*/
	void setBeta( float beta );

	/**
	* sets the emhasize parameter to all slines
	* @param emph is the new value of the emphasize parameter
	*/
	void setEmphasize( float emph );

	/**
	* computes splines
	* @param relations is a pointer to the vector of the relationships
	*/
	void createSplineRepresentation( std::vector< strRelationshipPair > *relations );

	/**
	* does the intersections tests
	* @param selective_a the on/off state of the spline
	* @param line is a pointer to line coordinates
	* @param zoom is a float encapsulating the zoom factor
	* @param pan is a pointer to the vector containing the pan offset
	*/
	void hasIntersections( bool selective_a, float *line, float zoom, VVector *pan );

	/**
	* does the intersections tests for inverse rooted trees extra
	* @param selective_a the on/off state of the spline
	* @param line is a pointer to line coordinates
	* @param zoom is a float encapsulating the zoom factor
	* @param pan is a pointer to the vector containing the pan offset
	*/
	void hasIntersectionsSpecial( bool selective_a, float *line, float zoom, VVector *pan );

	/**
	* draw splines (normal)
	*/
	void drawSplinesNormal(  );

	/**
	* draw splines (special - like for inverted trees or so)
	*/
	void drawSplinesSpecial(  );

	/**
	 * draws a radial tree
	 */
	void drawRadial();

	/**
	 * draws a inverted radial tree
	 */
	void drawInvRadial();

	/**
	 * draws a rooted tree
	 */
	void drawRooted();

	/**
	 * draws a inverted rooted tree
	 */
	void drawInvRooted();

	/**
	* computes a balloon tree
	*/
	void computeBalloon();

	/**
	* draws a balloon tree
	*/
	void drawBalloon();

	tree<TreeEntity> m_TreeDatastructure;	/**< abstract tree data structure. */
	tree<TreeEntity>::iterator m_TopNode;				/**< the root node of the abstract tree data structure. */
	
	int m_iNumHierarchies;					/**< number of hierarchies. */
	
	bool m_bDataPresent;					/**< states if the data is present. */
	bool m_bRelationsPresent;				/**< states if the relation data is present. */

	VVector *m_vPointColor;					/**< encapsulates the point color. */
	VVector *m_vLineColor;					/**< encapsulates the line color. */

	VVector *m_vSplineColor_1;				/**< encapsulates the spline starting color. */
	VVector *m_vSplineColor_2;				/**< encapsulates the spline end color. */

	float pointsize;						/**< encapsulates the actual pointsize. */
	float linewidth;						/**< encapsulates the actual line width. */

	float m_InitPointsize;					/**< encapsulates the initial pointsize. */
	float m_InitLinewidth;					/**< encapsulates the initial line width. */

	float m_fBeta;							/**< encapsulates the beta parameter. */
	float m_fAlpha;							/**< encapsulates the alpha parameter. */
	float m_fEmphasize;						/**< encapsulates the emphasize parameter. */
	
	std::vector< Spline* > splines;			/**< vector that encapsulates the splines rep. */

	
};

#endif
