#pragma once

#define NO_MIN_MAX

#include <math.h>
#include "VCommon.h"
#include "VVector.h"

// 4x4 Matrix class
class VMatrix
{
public:

	VMatrix()
	{
		m_vfElements.m_vfElements16[0]=m_vfElements.m_vfElements16[5]=m_vfElements.m_vfElements16[10]=m_vfElements.m_vfElements16[15]=1.0f;
		m_vfElements.m_vfElements16[1]=m_vfElements.m_vfElements16[2]=m_vfElements.m_vfElements16[3]=m_vfElements.m_vfElements16[4]=m_vfElements.m_vfElements16[6]=m_vfElements.m_vfElements16[7]=m_vfElements.m_vfElements16[8]=m_vfElements.m_vfElements16[9]=m_vfElements.m_vfElements16[11]=m_vfElements.m_vfElements16[12]=m_vfElements.m_vfElements16[13]=m_vfElements.m_vfElements16[14]=0.0f;
	};

	VMatrix(	const float & fElement0, const float & fElement4, const float & fElement8, const float & fElement12,
		const float & fElement1, const float & fElement5, const float & fElement9, const float & fElement13,
		const float & fElement2, const float & fElement6, const float & fElement10, const float & fElement14,
		const float & fElement3, const float & fElement7, const float & fElement11, const float & fElement15)
	{
		m_vfElements.m_vfElements16[0]=fElement0; m_vfElements.m_vfElements16[4]=fElement4;  m_vfElements.m_vfElements16[8]=fElement8;  m_vfElements.m_vfElements16[12]=fElement12;
		m_vfElements.m_vfElements16[1]=fElement1; m_vfElements.m_vfElements16[5]=fElement5;  m_vfElements.m_vfElements16[9]=fElement9;  m_vfElements.m_vfElements16[13]=fElement13;
		m_vfElements.m_vfElements16[2]=fElement2; m_vfElements.m_vfElements16[6]=fElement6; m_vfElements.m_vfElements16[10]=fElement10; m_vfElements.m_vfElements16[14]=fElement14;
		m_vfElements.m_vfElements16[3]=fElement3; m_vfElements.m_vfElements16[7]=fElement7; m_vfElements.m_vfElements16[11]=fElement11; m_vfElements.m_vfElements16[15]=fElement15;
	};

	VMatrix(const VVector & vecOne, const VVector & vecTwo, const VVector & vecThree, const VVector & vecFour = VVector(0.0f,0.0f,0.0f))
	{

		m_vfElements.m_vfElements16[0]=vecOne.getX(); m_vfElements.m_vfElements16[4]=vecOne.getY();  m_vfElements.m_vfElements16[8]=vecOne.getZ();  m_vfElements.m_vfElements16[12]=0.0f;
		m_vfElements.m_vfElements16[1]=vecTwo.getX(); m_vfElements.m_vfElements16[5]=vecTwo.getY();  m_vfElements.m_vfElements16[9]=vecTwo.getZ();  m_vfElements.m_vfElements16[13]=0.0f;
		m_vfElements.m_vfElements16[2]=vecThree.getX(); m_vfElements.m_vfElements16[6]=vecThree.getY(); m_vfElements.m_vfElements16[10]=vecThree.getZ(); m_vfElements.m_vfElements16[14]=0.0f;
		m_vfElements.m_vfElements16[3]=0.0f; m_vfElements.m_vfElements16[7]=0.0f; m_vfElements.m_vfElements16[11]=0.0f; m_vfElements.m_vfElements16[15]=1.0f;	
	};

	VMatrix(const float *pElements)
	{
		memcpy((void*)m_vfElements.m_vfElements16, (void *)pElements, 16*sizeof(float));
	};

	VMatrix(const float vfElements[4][4])
	{
		memcpy((void*)m_vfElements.m_vfElements16, (void *)vfElements, 16*sizeof(float));
	};


	VMatrix(const VVector axis, float angle)
	{
		float radangle = (angle) * 3.1415926535f / 180.0f;
		float rcos = cos(radangle);
		float rsin = sin(radangle);
		m_vfElements.m_vfElements44[0][0] =           rcos + axis.getX() * axis.getX() * (1.0f - rcos);
		m_vfElements.m_vfElements44[1][0] =  axis.getZ() * rsin + axis.getY() * axis.getX() * (1.0f - rcos);
		m_vfElements.m_vfElements44[2][0] = -axis.getY() * rsin + axis.getZ() * axis.getX() * (1.0f - rcos);
		m_vfElements.m_vfElements44[3][0] = 0.0f;
		m_vfElements.m_vfElements44[0][1] = -axis.getZ() * rsin + axis.getX() * axis.getY() * (1.0f - rcos);
		m_vfElements.m_vfElements44[1][1] =           rcos + axis.getY() * axis.getY() * (1.0f - rcos);
		m_vfElements.m_vfElements44[2][1] =  axis.getX() * rsin + axis.getZ() * axis.getY() * (1.0f - rcos);
		m_vfElements.m_vfElements44[3][1] = 0.0f;
		m_vfElements.m_vfElements44[0][2] =  axis.getY() * rsin + axis.getX() * axis.getZ() * (1.0f - rcos);
		m_vfElements.m_vfElements44[1][2] = -axis.getX() * rsin + axis.getY() * axis.getZ() * (1.0f - rcos);
		m_vfElements.m_vfElements44[2][2] =           rcos + axis.getZ() * axis.getZ() * (1.0f - rcos);
		m_vfElements.m_vfElements44[3][2] = 0.0f;
		m_vfElements.m_vfElements44[0][3] = 0.0f;
		m_vfElements.m_vfElements44[1][3] = 0.0f;
		m_vfElements.m_vfElements44[2][3] = 0.0f;
		m_vfElements.m_vfElements44[3][3] = 1.0f;
	}

	~VMatrix()
	{
	};

	void set(const float *pElements)
	{
		memcpy((void*)m_vfElements.m_vfElements16, (void *)pElements, 16*sizeof(float));
	};

	void clearMatrix()
	{
		m_vfElements.m_vfElements16[0]=m_vfElements.m_vfElements16[5]=m_vfElements.m_vfElements16[10]=m_vfElements.m_vfElements16[15]=1.0f;
		m_vfElements.m_vfElements16[1]=m_vfElements.m_vfElements16[2]=m_vfElements.m_vfElements16[3]=m_vfElements.m_vfElements16[4]=m_vfElements.m_vfElements16[6]=m_vfElements.m_vfElements16[7]=m_vfElements.m_vfElements16[8]=m_vfElements.m_vfElements16[9]=m_vfElements.m_vfElements16[11]=m_vfElements.m_vfElements16[12]=m_vfElements.m_vfElements16[13]=m_vfElements.m_vfElements16[14]=0.0f;
	};

	const float * get() const
	{
		return (float *) m_vfElements.m_vfElements16;
	};

	void set(const unsigned int i, const unsigned int j, const float & fValue)
	{
		m_vfElements.m_vfElements44[i][j] = fValue;
	};

	const float & get(const unsigned int i, const unsigned int j) const
	{
		return m_vfElements.m_vfElements44[i][j];
	};

	const float & get(const unsigned int i) const
	{
		return m_vfElements.m_vfElements16[i];
	};

	void setRow(const unsigned int i, const float vfValues[4])
	{
		m_vfElements.m_vfElements16[ i+0] = vfValues[0];
		m_vfElements.m_vfElements16[ i+4] = vfValues[1];
		m_vfElements.m_vfElements16[ i+8] = vfValues[2];
		m_vfElements.m_vfElements16[i+12] = vfValues[3];	
	};

	void setColumn(const unsigned int i, const float vfValues[4])
	{
		m_vfElements.m_vfElements16[i*4+0] = vfValues[0];
		m_vfElements.m_vfElements16[i*4+1] = vfValues[1];
		m_vfElements.m_vfElements16[i*4+2] = vfValues[2];
		m_vfElements.m_vfElements16[i*4+3] = vfValues[3];
	};

	void setRow(const unsigned int i, const VVector & vecVector)
	{
		m_vfElements.m_vfElements16[ i+0] = vecVector.getX();
		m_vfElements.m_vfElements16[ i+4] = vecVector.getY();
		m_vfElements.m_vfElements16[ i+8] = vecVector.getZ();
	};

	void setColumn(const unsigned int i, const VVector & vecVector)
	{
		m_vfElements.m_vfElements16[i*4+0] = vecVector.getX();
		m_vfElements.m_vfElements16[i*4+1] = vecVector.getY();
		m_vfElements.m_vfElements16[i*4+2] = vecVector.getZ();
	};

	float & operator()(const unsigned int i, const unsigned int j)
	{
		return m_vfElements.m_vfElements44[i][j];
	};

	const float & operator()(const unsigned int i, const unsigned int j) const
	{
		return m_vfElements.m_vfElements44[i][j];
	};

	void setTranslation(const VVector & vecTranslation)
	{
		m_vfElements.m_vfElements16[12] = vecTranslation.getX();
		m_vfElements.m_vfElements16[13] = vecTranslation.getY();
		m_vfElements.m_vfElements16[14] = vecTranslation.getZ();
		m_vfElements.m_vfElements16[15] = 1.0f;
	};

	const VVector getTranslation() const
	{
		return VVector(m_vfElements.m_vfElements16[12],m_vfElements.m_vfElements16[13],m_vfElements.m_vfElements16[14]);
	};

	const VMatrix getTranslated(const VVector & vecTranslation) const
	{
		return (*this) * VMatrix(1.0f,0.0f,0.0f,vecTranslation.getX(), 0.0f,1.0f,0.0f,vecTranslation.getY(), 0.0f,0.0f,1.0f,vecTranslation.getZ(), 0.0f, 0.0f,0.0f,1.0f);
	};

	void translate(const VVector &vecTranslation)
	{
		*this = getTranslated(vecTranslation);
	};

	const VMatrix getScaled(const VVector &vecScale) const
	{
		return (*this) * 	VMatrix(	vecScale.getX(),0.0f,0.0f,0.0f, 0.0f,vecScale.getY(),0.0f,0.0f,	0.0f,0.0f,vecScale.getZ(),0.0f, 0.0f,0.0f,0.0f,1.0f);
	};

	void scale(const VVector &vecScale)
	{
		*this = getScaled(vecScale);
	};

	const VMatrix getInverse() const
	{
		const float fTmp1 = m_vfElements.m_vfElements44[2][2]*m_vfElements.m_vfElements44[3][3];
		const float fTmp2 = m_vfElements.m_vfElements44[0][1]*fTmp1;
		const float fTmp4 = m_vfElements.m_vfElements44[2][3]*m_vfElements.m_vfElements44[3][2];
		const float fTmp5 = m_vfElements.m_vfElements44[0][1]*fTmp4;
		const float fTmp6 = m_vfElements.m_vfElements44[0][2]*m_vfElements.m_vfElements44[3][3];
		const float fTmp7 = m_vfElements.m_vfElements44[2][1]*fTmp6;
		const float fTmp8 = m_vfElements.m_vfElements44[0][3]*m_vfElements.m_vfElements44[3][2];
		const float fTmp9 = m_vfElements.m_vfElements44[2][1]*fTmp8;
		const float fTmp11 = m_vfElements.m_vfElements44[0][2]*m_vfElements.m_vfElements44[2][3];
		const float fTmp12 = m_vfElements.m_vfElements44[3][1]*fTmp11;
		const float fTmp14 = m_vfElements.m_vfElements44[0][3]*m_vfElements.m_vfElements44[2][2];
		const float fTmp15 = m_vfElements.m_vfElements44[3][1]*fTmp14;
		const float fTmp17 = m_vfElements.m_vfElements44[1][1]*fTmp1;
		const float fTmp19 = m_vfElements.m_vfElements44[1][1]*fTmp4;
		const float fTmp22 = m_vfElements.m_vfElements44[1][2]*m_vfElements.m_vfElements44[3][3];
		const float fTmp23 = m_vfElements.m_vfElements44[2][1]*fTmp22;
		const float fTmp26 = m_vfElements.m_vfElements44[1][3]*m_vfElements.m_vfElements44[3][2];
		const float fTmp27 = m_vfElements.m_vfElements44[2][1]*fTmp26;
		const float fTmp29 = m_vfElements.m_vfElements44[1][2]*m_vfElements.m_vfElements44[2][3];
		const float fTmp30 = m_vfElements.m_vfElements44[3][1]*fTmp29;
		const float fTmp32 = m_vfElements.m_vfElements44[1][3]*m_vfElements.m_vfElements44[2][2];
		const float fTmp33 = m_vfElements.m_vfElements44[3][1]*fTmp32;
		const float fTmp48 = m_vfElements.m_vfElements44[0][1]*fTmp22;
		const float fTmp50 = m_vfElements.m_vfElements44[0][1]*fTmp26;
		const float fTmp53 = m_vfElements.m_vfElements44[1][1]*fTmp6;
		const float fTmp56 = m_vfElements.m_vfElements44[1][1]*fTmp8;
		const float fTmp58 = m_vfElements.m_vfElements44[0][2]*m_vfElements.m_vfElements44[1][3];
		const float fTmp59 = m_vfElements.m_vfElements44[3][1]*fTmp58;
		const float fTmp61 = m_vfElements.m_vfElements44[0][3]*m_vfElements.m_vfElements44[1][2];
		const float fTmp62 = m_vfElements.m_vfElements44[3][1]*fTmp61;
		const float fTmp66 = m_vfElements.m_vfElements44[0][1]*fTmp29;
		const float fTmp69 = m_vfElements.m_vfElements44[0][1]*fTmp32;
		const float fTmp71 = m_vfElements.m_vfElements44[1][1]*fTmp11;
		const float fTmp73 = m_vfElements.m_vfElements44[1][1]*fTmp14;
		const float fTmp76 = m_vfElements.m_vfElements44[2][1]*fTmp58;
		const float fTmp79 = m_vfElements.m_vfElements44[2][1]*fTmp61;
		const float fTmp84 = 1.0f/(m_vfElements.m_vfElements44[0][0]*fTmp17-m_vfElements.m_vfElements44[0][0]*fTmp19-m_vfElements.m_vfElements44[0][0]*fTmp23+m_vfElements.m_vfElements44[0][0]*fTmp27+m_vfElements.m_vfElements44[0][0]*fTmp30-m_vfElements.m_vfElements44[0][0]*fTmp33-m_vfElements.m_vfElements44[1][0]*fTmp2+m_vfElements.m_vfElements44[1][0]*fTmp5+m_vfElements.m_vfElements44[1][0]*fTmp7-m_vfElements.m_vfElements44[1][0]*fTmp9-m_vfElements.m_vfElements44[1][0]*fTmp12+m_vfElements.m_vfElements44[1][0]*fTmp15+m_vfElements.m_vfElements44[2][0]*fTmp48-m_vfElements.m_vfElements44[2][0]*fTmp50-m_vfElements.m_vfElements44[2][0]*fTmp53+m_vfElements.m_vfElements44[2][0]*fTmp56+m_vfElements.m_vfElements44[2][0]*fTmp59-m_vfElements.m_vfElements44[2][0]*fTmp62-m_vfElements.m_vfElements44[3][0]*fTmp66+m_vfElements.m_vfElements44[3][0]*fTmp69+m_vfElements.m_vfElements44[3][0]*fTmp71-m_vfElements.m_vfElements44[3][0]*fTmp73-m_vfElements.m_vfElements44[3][0]*fTmp76+m_vfElements.m_vfElements44[3][0]*fTmp79);
		const float fTmp116 = m_vfElements.m_vfElements44[2][1]*m_vfElements.m_vfElements44[3][2];
		const float fTmp118 = m_vfElements.m_vfElements44[2][2]*m_vfElements.m_vfElements44[3][1];
		const float fTmp121 = m_vfElements.m_vfElements44[0][1]*m_vfElements.m_vfElements44[3][2];
		const float fTmp124 = m_vfElements.m_vfElements44[0][2]*m_vfElements.m_vfElements44[3][1];
		const float fTmp126 = m_vfElements.m_vfElements44[0][1]*m_vfElements.m_vfElements44[2][2];
		const float fTmp128 = m_vfElements.m_vfElements44[0][2]*m_vfElements.m_vfElements44[2][1];
		const float fTmp133 = m_vfElements.m_vfElements44[2][1]*m_vfElements.m_vfElements44[3][3];
		const float fTmp135 = m_vfElements.m_vfElements44[2][3]*m_vfElements.m_vfElements44[3][1];
		const float fTmp138 = m_vfElements.m_vfElements44[0][1]*m_vfElements.m_vfElements44[3][3];
		const float fTmp141 = m_vfElements.m_vfElements44[0][3]*m_vfElements.m_vfElements44[3][1];
		const float fTmp143 = m_vfElements.m_vfElements44[0][1]*m_vfElements.m_vfElements44[2][3];
		const float fTmp145 = m_vfElements.m_vfElements44[0][3]*m_vfElements.m_vfElements44[2][1];
		const float fTmp151 = m_vfElements.m_vfElements44[1][1]*m_vfElements.m_vfElements44[2][2];
		const float fTmp153 = m_vfElements.m_vfElements44[1][2]*m_vfElements.m_vfElements44[2][1];
		const float fTmp159 = m_vfElements.m_vfElements44[0][1]*m_vfElements.m_vfElements44[1][2];
		const float fTmp161 = m_vfElements.m_vfElements44[0][2]*m_vfElements.m_vfElements44[1][1];
		const float fTmp166 = m_vfElements.m_vfElements44[1][1]*m_vfElements.m_vfElements44[3][3];
		const float fTmp168 = m_vfElements.m_vfElements44[1][3]*m_vfElements.m_vfElements44[3][1];
		const float fTmp174 = m_vfElements.m_vfElements44[0][1]*m_vfElements.m_vfElements44[1][3];
		const float fTmp176 = m_vfElements.m_vfElements44[0][3]*m_vfElements.m_vfElements44[1][1];
		const float fTmp187 = m_vfElements.m_vfElements44[1][1]*m_vfElements.m_vfElements44[2][3];
		const float fTmp189 = m_vfElements.m_vfElements44[1][3]*m_vfElements.m_vfElements44[2][1];
		const float fTmp212 = m_vfElements.m_vfElements44[1][1]*m_vfElements.m_vfElements44[3][2];
		const float fTmp214 = m_vfElements.m_vfElements44[1][2]*m_vfElements.m_vfElements44[3][1];

		VMatrix matNew;
		matNew.m_vfElements.m_vfElements44[0][1] = (-fTmp2+fTmp5+fTmp7-fTmp9-fTmp12+fTmp15)*fTmp84;
		matNew.m_vfElements.m_vfElements44[0][2] = -(-fTmp48+fTmp50+fTmp53-fTmp56-fTmp59+fTmp62)*fTmp84;
		matNew.m_vfElements.m_vfElements44[1][2] = -(m_vfElements.m_vfElements44[0][0]*fTmp22-m_vfElements.m_vfElements44[0][0]*fTmp26-m_vfElements.m_vfElements44[1][0]*fTmp6+m_vfElements.m_vfElements44[1][0]*fTmp8+m_vfElements.m_vfElements44[3][0]*fTmp58-m_vfElements.m_vfElements44[3][0]*fTmp61)*fTmp84;
		matNew.m_vfElements.m_vfElements44[1][3] = -(-m_vfElements.m_vfElements44[0][0]*fTmp29+m_vfElements.m_vfElements44[0][0]*fTmp32+m_vfElements.m_vfElements44[1][0]*fTmp11-m_vfElements.m_vfElements44[1][0]*fTmp14-m_vfElements.m_vfElements44[2][0]*fTmp58+m_vfElements.m_vfElements44[2][0]*fTmp61)*fTmp84;
		matNew.m_vfElements.m_vfElements44[3][1] = (m_vfElements.m_vfElements44[0][0]*fTmp116-m_vfElements.m_vfElements44[0][0]*fTmp118-m_vfElements.m_vfElements44[2][0]*fTmp121+m_vfElements.m_vfElements44[2][0]*fTmp124+m_vfElements.m_vfElements44[3][0]*fTmp126-m_vfElements.m_vfElements44[3][0]*fTmp128)*fTmp84;
		matNew.m_vfElements.m_vfElements44[2][1] = -(m_vfElements.m_vfElements44[0][0]*fTmp133-m_vfElements.m_vfElements44[0][0]*fTmp135-m_vfElements.m_vfElements44[2][0]*fTmp138+m_vfElements.m_vfElements44[2][0]*fTmp141+m_vfElements.m_vfElements44[3][0]*fTmp143-m_vfElements.m_vfElements44[3][0]*fTmp145)*fTmp84;
		matNew.m_vfElements.m_vfElements44[3][3] = (m_vfElements.m_vfElements44[0][0]*fTmp151-m_vfElements.m_vfElements44[0][0]*fTmp153-m_vfElements.m_vfElements44[1][0]*fTmp126+m_vfElements.m_vfElements44[1][0]*fTmp128+m_vfElements.m_vfElements44[2][0]*fTmp159-m_vfElements.m_vfElements44[2][0]*fTmp161)*fTmp84;
		matNew.m_vfElements.m_vfElements44[2][2] = (m_vfElements.m_vfElements44[0][0]*fTmp166-m_vfElements.m_vfElements44[0][0]*fTmp168-m_vfElements.m_vfElements44[1][0]*fTmp138+m_vfElements.m_vfElements44[1][0]*fTmp141+m_vfElements.m_vfElements44[3][0]*fTmp174-m_vfElements.m_vfElements44[3][0]*fTmp176)*fTmp84;
		matNew.m_vfElements.m_vfElements44[2][0] = (m_vfElements.m_vfElements44[1][0]*fTmp133-m_vfElements.m_vfElements44[1][0]*fTmp135-m_vfElements.m_vfElements44[2][0]*fTmp166+m_vfElements.m_vfElements44[2][0]*fTmp168+m_vfElements.m_vfElements44[3][0]*fTmp187-m_vfElements.m_vfElements44[3][0]*fTmp189)*fTmp84;
		matNew.m_vfElements.m_vfElements44[1][0] = -(m_vfElements.m_vfElements44[1][0]*fTmp1-m_vfElements.m_vfElements44[1][0]*fTmp4-m_vfElements.m_vfElements44[2][0]*fTmp22+m_vfElements.m_vfElements44[2][0]*fTmp26+m_vfElements.m_vfElements44[3][0]*fTmp29-m_vfElements.m_vfElements44[3][0]*fTmp32)*fTmp84;
		matNew.m_vfElements.m_vfElements44[0][3] = -(fTmp66-fTmp69-fTmp71+fTmp73+fTmp76-fTmp79)*fTmp84;
		matNew.m_vfElements.m_vfElements44[3][2] = -(m_vfElements.m_vfElements44[0][0]*fTmp212-m_vfElements.m_vfElements44[0][0]*fTmp214-m_vfElements.m_vfElements44[1][0]*fTmp121+m_vfElements.m_vfElements44[1][0]*fTmp124+m_vfElements.m_vfElements44[3][0]*fTmp159-m_vfElements.m_vfElements44[3][0]*fTmp161)*fTmp84;
		matNew.m_vfElements.m_vfElements44[0][0] = (fTmp17-fTmp19-fTmp23+fTmp27+fTmp30-fTmp33)*fTmp84;
		matNew.m_vfElements.m_vfElements44[2][3] = -(m_vfElements.m_vfElements44[0][0]*fTmp187-m_vfElements.m_vfElements44[0][0]*fTmp189-m_vfElements.m_vfElements44[1][0]*fTmp143+m_vfElements.m_vfElements44[1][0]*fTmp145+m_vfElements.m_vfElements44[2][0]*fTmp174-m_vfElements.m_vfElements44[2][0]*fTmp176)*fTmp84;
		matNew.m_vfElements.m_vfElements44[3][0] = -(m_vfElements.m_vfElements44[1][0]*fTmp116-m_vfElements.m_vfElements44[1][0]*fTmp118-m_vfElements.m_vfElements44[2][0]*fTmp212+m_vfElements.m_vfElements44[2][0]*fTmp214+m_vfElements.m_vfElements44[3][0]*fTmp151-m_vfElements.m_vfElements44[3][0]*fTmp153)*fTmp84;
		matNew.m_vfElements.m_vfElements44[1][1] = (m_vfElements.m_vfElements44[0][0]*fTmp1-m_vfElements.m_vfElements44[0][0]*fTmp4-m_vfElements.m_vfElements44[2][0]*fTmp6+m_vfElements.m_vfElements44[2][0]*fTmp8+m_vfElements.m_vfElements44[3][0]*fTmp11-m_vfElements.m_vfElements44[3][0]*fTmp14)*fTmp84;

		return matNew;
	};

	void invert()
	{
		(*this) = getInverse();
	}

	const VMatrix getTransposed() const
	{
		return VMatrix(
			m_vfElements.m_vfElements16[0],m_vfElements.m_vfElements16[1],m_vfElements.m_vfElements16[2],m_vfElements.m_vfElements16[3],
			m_vfElements.m_vfElements16[4],m_vfElements.m_vfElements16[5],m_vfElements.m_vfElements16[6],m_vfElements.m_vfElements16[7],
			m_vfElements.m_vfElements16[8],m_vfElements.m_vfElements16[9],m_vfElements.m_vfElements16[10],m_vfElements.m_vfElements16[11],
			m_vfElements.m_vfElements16[12],m_vfElements.m_vfElements16[13],m_vfElements.m_vfElements16[14],m_vfElements.m_vfElements16[15]);
	};

	void transpose()
	{
		(*this) = getTransposed();
	};

	VVector homogenTransform(const VVector& v)
	{
		float vector[4];
		vector[0] = v.getX();
		vector[1] = v.getY();
		vector[2] = v.getZ();
		vector[3] = 1.0f;

		float result[4];

		transform(vector, result);
		
		for(int i = 0; i < 3;++i)
		{
			result[i] /= result[3];
		}

		VVector res(result[0], result[1], result[2]);
		return res;
	}

	void transform(float * v, float * o)
	{
		o[0]=m_vfElements.m_vfElements16[0]*v[0]+m_vfElements.m_vfElements16[4]*v[1]+m_vfElements.m_vfElements16[8 ]*v[2]+m_vfElements.m_vfElements16[12]*v[3];
		o[1]=m_vfElements.m_vfElements16[1]*v[0]+m_vfElements.m_vfElements16[5]*v[1]+m_vfElements.m_vfElements16[9 ]*v[2]+m_vfElements.m_vfElements16[13]*v[3];
		o[2]=m_vfElements.m_vfElements16[2]*v[0]+m_vfElements.m_vfElements16[6]*v[1]+m_vfElements.m_vfElements16[10]*v[2]+m_vfElements.m_vfElements16[14]*v[3];
		o[3]=m_vfElements.m_vfElements16[3]*v[0]+m_vfElements.m_vfElements16[7]*v[1]+m_vfElements.m_vfElements16[11]*v[2]+m_vfElements.m_vfElements16[15]*v[3];
	}

	const VVector getRotated(const VVector & vecOther) const
	{
		return VVector(
			(m_vfElements.m_vfElements16[0]*vecOther.getX() + m_vfElements.m_vfElements16[4]*vecOther.getY() + m_vfElements.m_vfElements16[8]*vecOther.getZ()), 
			(m_vfElements.m_vfElements16[1]*vecOther.getX() + m_vfElements.m_vfElements16[5]*vecOther.getY() + m_vfElements.m_vfElements16[9]*vecOther.getZ()),
			(m_vfElements.m_vfElements16[2]*vecOther.getX() + m_vfElements.m_vfElements16[6]*vecOther.getY() + m_vfElements.m_vfElements16[10]*vecOther.getZ()));
	};

	const bool operator==(const VMatrix & matOther) const
	{
		for (unsigned int i=0;i < 16; i++)
		{
			if (m_vfElements.m_vfElements16[i] != matOther.m_vfElements.m_vfElements16[i])
				return false;
		}

		return true;
	};

	const bool operator!=(const VMatrix & matOther) const
	{
		return !(*this == matOther);
	};

	const VMatrix operator*(const VMatrix & matOther) const
	{
		return VMatrix(
			m_vfElements.m_vfElements16[0]*matOther.m_vfElements.m_vfElements16[0] + m_vfElements.m_vfElements16[4]*matOther.m_vfElements.m_vfElements16[1] + m_vfElements.m_vfElements16[8]*matOther.m_vfElements.m_vfElements16[2] + m_vfElements.m_vfElements16[12]*matOther.m_vfElements.m_vfElements16[3],
			m_vfElements.m_vfElements16[0]*matOther.m_vfElements.m_vfElements16[4] + m_vfElements.m_vfElements16[4]*matOther.m_vfElements.m_vfElements16[5] + m_vfElements.m_vfElements16[8]*matOther.m_vfElements.m_vfElements16[6] + m_vfElements.m_vfElements16[12]*matOther.m_vfElements.m_vfElements16[7],
			m_vfElements.m_vfElements16[0]*matOther.m_vfElements.m_vfElements16[8] + m_vfElements.m_vfElements16[4]*matOther.m_vfElements.m_vfElements16[9] + m_vfElements.m_vfElements16[8]*matOther.m_vfElements.m_vfElements16[10] + m_vfElements.m_vfElements16[12]*matOther.m_vfElements.m_vfElements16[11],
			m_vfElements.m_vfElements16[0]*matOther.m_vfElements.m_vfElements16[12] + m_vfElements.m_vfElements16[4]*matOther.m_vfElements.m_vfElements16[13] + m_vfElements.m_vfElements16[8]*matOther.m_vfElements.m_vfElements16[14] + m_vfElements.m_vfElements16[12]*matOther.m_vfElements.m_vfElements16[15],

			m_vfElements.m_vfElements16[1]*matOther.m_vfElements.m_vfElements16[0] + m_vfElements.m_vfElements16[5]*matOther.m_vfElements.m_vfElements16[1] + m_vfElements.m_vfElements16[9]*matOther.m_vfElements.m_vfElements16[2] + m_vfElements.m_vfElements16[13]*matOther.m_vfElements.m_vfElements16[3],
			m_vfElements.m_vfElements16[1]*matOther.m_vfElements.m_vfElements16[4] + m_vfElements.m_vfElements16[5]*matOther.m_vfElements.m_vfElements16[5] + m_vfElements.m_vfElements16[9]*matOther.m_vfElements.m_vfElements16[6] + m_vfElements.m_vfElements16[13]*matOther.m_vfElements.m_vfElements16[7],
			m_vfElements.m_vfElements16[1]*matOther.m_vfElements.m_vfElements16[8] + m_vfElements.m_vfElements16[5]*matOther.m_vfElements.m_vfElements16[9] + m_vfElements.m_vfElements16[9]*matOther.m_vfElements.m_vfElements16[10] + m_vfElements.m_vfElements16[13]*matOther.m_vfElements.m_vfElements16[11],
			m_vfElements.m_vfElements16[1]*matOther.m_vfElements.m_vfElements16[12] + m_vfElements.m_vfElements16[5]*matOther.m_vfElements.m_vfElements16[13] + m_vfElements.m_vfElements16[9]*matOther.m_vfElements.m_vfElements16[14] + m_vfElements.m_vfElements16[13]*matOther.m_vfElements.m_vfElements16[15],

			m_vfElements.m_vfElements16[2]*matOther.m_vfElements.m_vfElements16[0] + m_vfElements.m_vfElements16[6]*matOther.m_vfElements.m_vfElements16[1] + m_vfElements.m_vfElements16[10]*matOther.m_vfElements.m_vfElements16[2] + m_vfElements.m_vfElements16[14]*matOther.m_vfElements.m_vfElements16[3],
			m_vfElements.m_vfElements16[2]*matOther.m_vfElements.m_vfElements16[4] + m_vfElements.m_vfElements16[6]*matOther.m_vfElements.m_vfElements16[5] + m_vfElements.m_vfElements16[10]*matOther.m_vfElements.m_vfElements16[6] + m_vfElements.m_vfElements16[14]*matOther.m_vfElements.m_vfElements16[7],
			m_vfElements.m_vfElements16[2]*matOther.m_vfElements.m_vfElements16[8] + m_vfElements.m_vfElements16[6]*matOther.m_vfElements.m_vfElements16[9] + m_vfElements.m_vfElements16[10]*matOther.m_vfElements.m_vfElements16[10] + m_vfElements.m_vfElements16[14]*matOther.m_vfElements.m_vfElements16[11],
			m_vfElements.m_vfElements16[2]*matOther.m_vfElements.m_vfElements16[12] + m_vfElements.m_vfElements16[6]*matOther.m_vfElements.m_vfElements16[13] + m_vfElements.m_vfElements16[10]*matOther.m_vfElements.m_vfElements16[14] + m_vfElements.m_vfElements16[14]*matOther.m_vfElements.m_vfElements16[15],

			m_vfElements.m_vfElements16[3]*matOther.m_vfElements.m_vfElements16[0] + m_vfElements.m_vfElements16[7]*matOther.m_vfElements.m_vfElements16[1] + m_vfElements.m_vfElements16[11]*matOther.m_vfElements.m_vfElements16[2] + m_vfElements.m_vfElements16[15]*matOther.m_vfElements.m_vfElements16[3],
			m_vfElements.m_vfElements16[3]*matOther.m_vfElements.m_vfElements16[4] + m_vfElements.m_vfElements16[7]*matOther.m_vfElements.m_vfElements16[5] + m_vfElements.m_vfElements16[11]*matOther.m_vfElements.m_vfElements16[6] + m_vfElements.m_vfElements16[15]*matOther.m_vfElements.m_vfElements16[7],
			m_vfElements.m_vfElements16[3]*matOther.m_vfElements.m_vfElements16[8] + m_vfElements.m_vfElements16[7]*matOther.m_vfElements.m_vfElements16[9] + m_vfElements.m_vfElements16[11]*matOther.m_vfElements.m_vfElements16[10] + m_vfElements.m_vfElements16[15]*matOther.m_vfElements.m_vfElements16[11],
			m_vfElements.m_vfElements16[3]*matOther.m_vfElements.m_vfElements16[12] + m_vfElements.m_vfElements16[7]*matOther.m_vfElements.m_vfElements16[13] + m_vfElements.m_vfElements16[11]*matOther.m_vfElements.m_vfElements16[14] + m_vfElements.m_vfElements16[15]*matOther.m_vfElements.m_vfElements16[15] );
	};

	const VMatrix & operator*=(const VMatrix & matOther)
	{
		*this = (*this) * matOther;
		return *this;
	};

	const VVector operator* (const VVector &vecOther) const
	{
		const float fW = m_vfElements.m_vfElements16[3]*vecOther.getX() + m_vfElements.m_vfElements16[7]*vecOther.getY() + m_vfElements.m_vfElements16[11]*vecOther.getZ() + m_vfElements.m_vfElements16[15];
		return VVector(
			(m_vfElements.m_vfElements16[0]*vecOther.getX() + m_vfElements.m_vfElements16[4]*vecOther.getY() + m_vfElements.m_vfElements16[8]*vecOther.getZ() + m_vfElements.m_vfElements16[12]) / fW, 
			(m_vfElements.m_vfElements16[1]*vecOther.getX() + m_vfElements.m_vfElements16[5]*vecOther.getY() + m_vfElements.m_vfElements16[9]*vecOther.getZ() + m_vfElements.m_vfElements16[13]) / fW,
			(m_vfElements.m_vfElements16[2]*vecOther.getX() + m_vfElements.m_vfElements16[6]*vecOther.getY() + m_vfElements.m_vfElements16[10]*vecOther.getZ() + m_vfElements.m_vfElements16[14]) / fW );
	};

	float& operator [] (unsigned int n)
	{
		return m_vfElements.m_vfElements16[n];
	}

	/** Construct a rotation matrix around the x axis. */
	static VMatrix RotationX(const float angleRad)
	{
		VMatrix m;

		float cosine = cos(angleRad);
		float sinus = sin(angleRad);

		//  |  1  0       0       0 |
		//M=|  0  cos(A) -sin(A)  0 |
		//  |  0  sin(A)  cos(A)  0 |
		//  |  0  0       0       1 |
		m[0]=1; m[4]=0;			m[8]=0;			m[12]=0;
		m[1]=0; m[5]=cosine;	m[9]=-sinus;	m[13]=0;
		m[2]=0; m[6]=sinus;		m[10]=cosine;	m[14]=0;
		m[3]=0; m[7]=0;         m[11]=0;        m[15]=1;
		return m;
	}

	/** Construct a rotation matrix around the x axis. */
	static VMatrix RotationY(const float angleRad)
	{
		VMatrix m;

		float cosine = cos(angleRad);
		float sinus = sin(angleRad);

		//  |  cos(A)  0  sin(A)  0 |
		//M=|  0       1  0       0 |
		//  | -sin(A)  0  cos(A)  0 |
		//  |  0       0  0       1 |
		m[0]=cosine;	m[4]=0; m[8]=sinus;		m[12]=0;
		m[1]=0;         m[5]=1; m[9]=0;         m[13]=0;
		m[2]=-sinus;	m[6]=0; m[10]=cosine;	m[14]=0;
		m[3]=0;         m[7]=0; m[11]=0;        m[15]=1;
		return m;
	}

	/** Construct a rotation matrix around the x axis. */
	static VMatrix RotationZ(const float angleRad)
	{
		VMatrix m;

		float cosine = cos(angleRad);
		float sinus = sin(angleRad);

		//  |  cos(A)  -sin(A) 0  0 |
		//M=|  sin(A)   cos(A) 0  0 |
		//  |  0        0      1  0 |
		//  |  0        0      0  1 |
		m[0]=cosine;	m[4]=-sinus;	m[8]=0;  m[12]=0;
		m[1]=sinus;		m[5]=cosine;	m[9]=0;  m[13]=0;
		m[2]=0;         m[6]=0;         m[10]=1; m[14]=0;
		m[3]=0;         m[7]=0;         m[11]=0; m[15]=1;
		return m;
	}

private:

	union
	{
		float m_vfElements16[16];
		float m_vfElements44[4][4];
	} m_vfElements;
};

inline std::ostream & operator << (std::ostream & os, const VMatrix & matMatrix)
{
	os	<< "((" << matMatrix.get(0,0) << ";" << matMatrix.get(0,1) << ";" << matMatrix.get(0,2) << ";" << matMatrix.get(0,3) << ")" << ";"
		<< "(" << matMatrix.get(1,0) << ";" << matMatrix.get(1,1) << ";" << matMatrix.get(1,2) << ";" << matMatrix.get(1,3) << ")" << ";"
		<< "(" << matMatrix.get(2,0) << ";" << matMatrix.get(2,1) << ";" << matMatrix.get(2,2) << ";" << matMatrix.get(2,3) << ")" << ";"
		<< "(" << matMatrix.get(3,0) << ";" << matMatrix.get(3,1) << ";" << matMatrix.get(3,2) << ";" << matMatrix.get(3,3) << "))";
	return os;
}

inline std::istream & operator>> (std::istream & is,  VMatrix & matMatrix)
{
	float f00, f01, f02, f03;
	float f10, f11, f12, f13;
	float f20, f21, f22, f23;
	float f30, f31, f32, f33;

	if (is 
		>> eat ("(")
		>> eat("(") >> f00 >> eat(";") >> f01 >> eat(";") >> f02 >> eat(";") >> f03 >> eat(")") >> eat(";")
		>> eat("(") >> f10 >> eat(";") >> f11 >> eat(";") >> f12 >> eat(";") >> f13 >> eat(")") >> eat(";")
		>> eat("(") >> f20 >> eat(";") >> f21 >> eat(";") >> f22 >> eat(";") >> f23 >> eat(")") >> eat(";")
		>> eat("(") >> f30 >> eat(";") >> f31 >> eat(";") >> f32 >> eat(";") >> f33 >> eat(")") 
		>> eat(")"))
	{
		matMatrix.set(0,0,f00);	matMatrix.set(0,1,f01);	matMatrix.set(0,2,f02);	matMatrix.set(0,3,f03);
		matMatrix.set(1,0,f10);	matMatrix.set(1,1,f11);	matMatrix.set(1,2,f12);	matMatrix.set(1,3,f13);
		matMatrix.set(2,0,f20);	matMatrix.set(2,1,f21);	matMatrix.set(2,2,f22);	matMatrix.set(2,3,f23);
		matMatrix.set(3,0,f30);	matMatrix.set(3,1,f31);	matMatrix.set(3,2,f32);	matMatrix.set(3,3,f33);

	}

	return is;
}