#include "QDoubleCanvasColor.h"

QDoubleCanvasColor::QDoubleCanvasColor(QWidget* parent /*= 0*/) : QObject(parent), QGLWidget(parent), m_bColorSet_1( false ), m_bColorSet_2( false )
{
	glInit ();
}


QDoubleCanvasColor::~QDoubleCanvasColor()
{

}

void QDoubleCanvasColor::setObjectName(const QString &name)
{
}

void QDoubleCanvasColor::setTooltip_1(QString s)
{
	m_qstrTooltip_1 = s;
}


void QDoubleCanvasColor::setTooltip_2(QString s)
{
	m_qstrTooltip_2 = s;
}

void QDoubleCanvasColor::setVolBackColorPtr_1(VVector *v)
{
	m_bColorSet_1 = true;
	m_ptrCanvasColor_1 = v;

	/*m_VolCanvasColor->setX(0.0f);
	m_VolCanvasColor->setY(0.0f);
	m_VolCanvasColor->setZ(0.0f);*/
}

void QDoubleCanvasColor::setVolBackColorPtr_2(VVector *v)
{
	m_bColorSet_2 = true;
	m_ptrCanvasColor_2 = v;

	/*m_VolCanvasColor->setX(0.0f);
	m_VolCanvasColor->setY(0.0f);
	m_VolCanvasColor->setZ(0.0f);*/
}


void QDoubleCanvasColor::initializeGL()
{
	this->makeCurrent();
	// Set up the rendering context, define display lists etc.:
	//...
	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	//...

	paintGL();
}

void QDoubleCanvasColor::resizeGL( int w, int h )
{
	this->makeCurrent();

	glViewport(0, 0, w, h);
	gluOrtho2D(0, w, h, 0);

}

void QDoubleCanvasColor::paintGL()
{	

	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);

	// draw the scene:
	glPushAttrib(GL_ALL_ATTRIB_BITS);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	glLoadIdentity();									// Reset The Current Modelview Matrix

	if( m_bColorSet_1 && m_bColorSet_2 ) 
	{
	
		glBegin( GL_QUADS );
		glColor3fv( m_ptrCanvasColor_1->getPtr() );
		glVertex2f( -1.0f, -1.0f );
		glVertex2f(  0.0f, -1.0f );
		glVertex2f(  0.0f,  1.0f );
		glVertex2f( -1.0f,  1.0f );

		glColor3fv( m_ptrCanvasColor_2->getPtr() );
		glVertex2f(  0.0f, -1.0f );
		glVertex2f(  1.0f, -1.0f );
		glVertex2f(  1.0f,  1.0f );
		glVertex2f(  0.0f,  1.0f );
		glEnd();

	}

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glPopAttrib();
}

void QDoubleCanvasColor::redraw()
{
	makeCurrent();
	repaint();
	swapBuffers();
}
void QDoubleCanvasColor::paintEvent( QPaintEvent *e )
{
	QGLWidget::paintEvent(e);
};

void QDoubleCanvasColor::mousePressEvent ( QMouseEvent * e )
{			
	const VMouseEvent me(getMouseEvent(e));
	VVector position = me.getPosition();

	QColor qcolor = QColorDialog::getColor();
	VVector color( ( float )qcolor.red()/255.0f, ( float )qcolor.green()/255.0f, ( float )qcolor.blue(  )/255.0f );

	if( position.getX() < 0.0 )
	{
		m_ptrCanvasColor_1->setX( color.getX() );
		m_ptrCanvasColor_1->setY( color.getY() );
		m_ptrCanvasColor_1->setZ( color.getZ() );
	}
	else
	{
		m_ptrCanvasColor_2->setX( color.getX() );
		m_ptrCanvasColor_2->setY( color.getY() );
		m_ptrCanvasColor_2->setZ( color.getZ() );
	}

	redraw();

};

void QDoubleCanvasColor::mouseReleaseEvent ( QMouseEvent * e )
{
	//const VMouseEvent me(getMouseEvent(e));

};

void QDoubleCanvasColor::mouseMoveEvent ( QMouseEvent * e )
{
	const VMouseEvent me(getMouseEvent(e));

	// v is the position of the mouse
	// cursor over canvas: v ranges in R, from -1 to 1 in both directions, left bottom corner is -1,-1
	// cursor n.o. canvas: if mouse was pressed before cursor left canvas, position is outside the range R 
	const VVector v  = me.getPosition();

	if ( v.getX() < 0 )
	{
		setToolTip( m_qstrTooltip_1 );
	}
	else
	{
		setToolTip( m_qstrTooltip_2 );
	}

	//std::cout << "VolRend: " << v << std::endl;

};

void QDoubleCanvasColor::mouseDoubleClickEvent ( QMouseEvent * e )
{

};

void QDoubleCanvasColor::keyPressEvent ( QKeyEvent * e )
{
	//const VKeyboardEvent ke(getKeyboardEvent(e));

};

void QDoubleCanvasColor::keyReleaseEvent ( QKeyEvent * e )
{
	//const VKeyboardEvent ke(getKeyboardEvent(e));

};

const VMouseEvent QDoubleCanvasColor::getMouseEvent (QMouseEvent *e)
{
	const VVector vecPosition( (2.0f * float(e->x()) - float(width()))  / float(width()),
		( float(height()) - 2.0f * float(e->y())) / float(height()),
		0.0f);
	const int iButton = 
		((e->button() == Qt::LeftButton)	? VMouseEvent::BUTTON_LEFT	 : 
		((e->button() == Qt::MidButton)		? VMouseEvent::BUTTON_MIDDLE : 
		((e->button() == Qt::RightButton)	? VMouseEvent::BUTTON_RIGHT	 : VMouseEvent::BUTTON_NONE)));

	const int iStateLeft	= (e->buttons () & Qt::LeftButton)	? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;
	const int iStateMiddle	= (e->buttons () & Qt::MidButton)	? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;
	const int iStateRight	= (e->buttons () & Qt::RightButton) ? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;

	const int iModifiers =   ((e->modifiers()   & Qt::ShiftModifier)   ? VKeyboardEvent::MODIFIER_SHIFT : 0) 
		| ((e->modifiers()	& Qt::ControlModifier) ? VKeyboardEvent::MODIFIER_CTRL  : 0) 
		| ((e->modifiers()	& Qt::AltModifier)     ? VKeyboardEvent::MODIFIER_ALT   : 0);

	return VMouseEvent(vecPosition,iButton,iStateLeft,iStateMiddle,iStateRight,iModifiers);

};

const VKeyboardEvent QDoubleCanvasColor::getKeyboardEvent (QKeyEvent *e)
{
	const int iKey = e->key();

	const int iModifiers =	  ((e->modifiers() & Qt::ShiftModifier)		? VKeyboardEvent::MODIFIER_SHIFT : 0) 
		| ((e->modifiers() & Qt::ControlModifier)	? VKeyboardEvent::MODIFIER_CTRL	 : 0) 
		| ((e->modifiers() & Qt::AltModifier)		? VKeyboardEvent::MODIFIER_ALT	 : 0);

	return VKeyboardEvent(iKey,iModifiers);

};

