#include "QMain.h"
#include "QtGui/QFileDialog.h"
#include <iostream>


QMain::QMain(QWidget* parent /*= 0*/, Qt::WFlags flags /*= 0*/) 
								: QMainWindow(parent, flags)
								
{
	setupUi(this);
	
	GLenum err = glewInit();

	if (err != GLEW_OK)
	{
		// glewInit failed, something is seriously wrong
		std::cerr << "Error initializing GLEW: " << glewGetErrorString(err) << std::endl;
		std::cout << "Press a button to continue" << std::endl;
		std::cin.get();
//		exit(-1);
	}
	std::cout << "- GLEW initialized." << std::endl << std::endl;

// 	ilInit();
// 	iluInit();
// 	ilutInit();
// 	ilutRenderer(ILUT_OPENGL);
// 
// 	if ((ilGetInteger(IL_VERSION_NUM) < IL_VERSION) ||
// 		(iluGetInteger(ILU_VERSION_NUM) < ILU_VERSION) ||
// 		(ilutGetInteger(ILUT_VERSION_NUM) < ILUT_VERSION)) 
// 	{
// 		std::cout << "devil not working" << std::endl;
// 		std::cout << "Press a button to continue" << std::endl;
// 		std::cin.get();
// 		exit(-1);
// 		return;
// 	}

	m_GraphColor_1.setX( 1.0f );
	m_GraphColor_1.setY( 1.0f );
	m_GraphColor_1.setZ( 1.0f );
	m_GraphColor_2.setX( 1.0f );
	m_GraphColor_2.setY( 1.0f );
	m_GraphColor_2.setZ( 1.0f );
	graphcolorcanvas->setVolBackColorPtr_1( &m_GraphColor_1 );
	graphcolorcanvas->setVolBackColorPtr_2( &m_GraphColor_2 );
	graphcolorcanvas->setTooltip_1( "Select graph edges color" );
	graphcolorcanvas->setTooltip_2( "Select graph nodes color" );

	m_SplineColor_1.setX( 0.0f );
	m_SplineColor_1.setY( 170.0f / 255.0f );
	m_SplineColor_1.setZ( 1.0f );
	m_SplineColor_2.setX( 1.0f );
	m_SplineColor_2.setY( 1.0f );
	m_SplineColor_2.setZ( 20.0f / 255.0f );
	edgescolorcanvas->setVolBackColorPtr_1( &m_SplineColor_1 );
	edgescolorcanvas->setVolBackColorPtr_2( &m_SplineColor_2 );
	edgescolorcanvas->setTooltip_1( "Select spline starting color" );
	edgescolorcanvas->setTooltip_2( "Select spline end color" );

	m_BackgroundColor.setX( 79.0f / 255.0f );
	m_BackgroundColor.setY( 79.0f / 255.0f );
	m_BackgroundColor.setZ( 79.0f / 255.0f );
	backgroundSelect->setVolBackColorPtr( &m_BackgroundColor );
	renderer_canvas->setBackColorPtr( &m_BackgroundColor );

	

	info_canvas->setVolBackColorPtr( &m_BackgroundColor );

	int _c_x2, _c_y2;
	renderer_canvas->geometry().getCoords( &pixposx_rcanvas, &pixposy_rcanvas, &_c_x2, &_c_y2 );

	w_sizex = size().rwidth();
	w_sizey = size().rheight();

	pixoffsetx_rcanvas = pixposx_rcanvas + ( (w_sizex -1) - _c_x2 );
	pixoffsety_rcanvas = pixposy_rcanvas + ( (w_sizey -1) - _c_y2 );

	pTimer = new QTimer( this);
	connect( pTimer, SIGNAL(timeout()), this, SLOT(watch()) );
	pTimer->start(0);

	pTimer_redraw = new QTimer( this);
	connect( pTimer_redraw, SIGNAL(timeout()), this, SLOT(redraw()) );
	pTimer_redraw->start(30);
	
	m_Tree = TreeRep();
	data_present = false;
	rel_present = false;

	splines_selstate = false;
	renderer_canvas->m_splinesel_active = &splines_selstate;

	setMinimumSize(w_sizex,w_sizey);

	renderer_canvas->name	= name_text;
	renderer_canvas->born	= born_text;
	renderer_canvas->died	= died_text;
	//renderer_canvas->foto = /*canvas*/;
	renderer_canvas->parents = parents_text;
	renderer_canvas->kids	= child_text;

	info_canvas->setVolBackColorPtr( &m_BackgroundColor );
	renderer_canvas->foto = info_canvas;

	//std::cout << (int)(-0.05) << std::endl;
	
	connect( actionOpen,				SIGNAL(triggered()),			this, SLOT(openDataFile()) );
	connect( actionLoadRelations,		SIGNAL(triggered()),			this, SLOT(openRealtionsFile()));
	connect( actionExit,				SIGNAL(triggered()),			this, SLOT(close()) );
	connect( radTreeButton,				SIGNAL(clicked(bool)),			this, SLOT(radtreeb_changed(bool)) );
	connect( invradTreeButton,			SIGNAL(clicked(bool)),			this, SLOT(invradtreeb_changed(bool)) );
	connect( rootedTreeButton,			SIGNAL(clicked(bool)),			this, SLOT(roottreeb_changed(bool)) );
	connect( invrootedTreeButton,		SIGNAL(clicked(bool)),			this, SLOT(invroottreeb_changed(bool)) );
	connect( balloonTreeButton,			SIGNAL(clicked(bool)),			this, SLOT(baltreeb_changed(bool)) );

	connect( beta_slider,				SIGNAL(valueChanged(int)),		this, SLOT(betaval_changed(int)) );
	connect( alpha_slider,				SIGNAL(valueChanged(int)),		this, SLOT(alphaval_changed(int)) );
	connect( emphasize_slider,			SIGNAL(valueChanged(int)),		this, SLOT(emphval_changed(int)) );
	
	connect( selection_cb,				SIGNAL(clicked(bool)),			this, SLOT(select_splines(bool)) );
	renderer_canvas->m_TreeRep = &m_Tree;

	
}


QMain::~QMain()
{

	delete pTimer;
}

void QMain::redraw()
{
	renderer_canvas->repaint();
}


void QMain::watch()
{

	int _w_sizex, _w_sizey;
	_w_sizex = size().rwidth();
	_w_sizey = size().rheight();

	if (_w_sizex != w_sizex || _w_sizey != w_sizey)
	{

		int _c_sizex, _c_sizey;
		_c_sizex = _w_sizex - pixoffsetx_rcanvas;
		_c_sizey = _w_sizey - pixoffsety_rcanvas;

		renderer_canvas->setGeometry(
			QRect(pixposx_rcanvas, pixposy_rcanvas, _c_sizex, _c_sizey));

		
		renderer_canvas->resizeGL( _c_sizex, _c_sizey );

		w_sizex = _w_sizex;
		w_sizey = _w_sizey;
	}


}

void QMain::openDataFile()
{
	std::cout << "load data" << std::endl;

	m_Tree.clear();

	name_text->setText( "" );
	born_text->setText( "" );
	died_text->setText( "" );
	parents_text->setText( "" );
	child_text->setText( "" );
	info_canvas->setTexHandlePtr( NULL );
	info_canvas->repaint();


	//file dialog, retrieve filename and path
	QString filename = QFileDialog::getOpenFileName(this,
		tr("Open Dataset"), ".", tr("Tree Data Files (*.tree)"));

	std::string tmp(filename.toAscii());

	if(tmp == "")
	{
		return;
	}

	if ( !( FamilyTreeParser::parse_treedata( &m_Tree, filename ) ) )
	{
		std::cerr << "Error loading file" << std::endl;
		return;
	}

	data_present = true;
	m_Tree.m_bDataPresent = data_present;

	m_Tree.m_vLineColor = &m_GraphColor_1;
	m_Tree.m_vPointColor = &m_GraphColor_2;
	m_Tree.m_vSplineColor_1 = &m_SplineColor_1;
	m_Tree.m_vSplineColor_2 = &m_SplineColor_2;

	if ( renderer_canvas->getLayout() == 4)
	{
		m_Tree.computeBalloon();
	}

}

void QMain::openRealtionsFile()
{
	if ( !data_present ) return;

	m_Tree.clear();

	std::cout << "load relations" << std::endl;

	//file dialog, retrieve filename and path
	QString filename = QFileDialog::getOpenFileName(this,
		tr("Open Relations Dataset"), ".", tr("Relationship Data Files (*.rel)"));

	if ( !( FamilyTreeParser::parse_relationdata( &relations, filename ) ) )
	{
		std::cerr << "Error loading file" << std::endl;
		return;
	}
	
	rel_present = true;
	m_Tree.m_bRelationsPresent = rel_present;
	
	//TODO: this is a dummy and has to be changed!!!
	m_Tree.createSplineRepresentation( &relations );

}

void QMain::close()
{
	//exit( 0 );
}

void QMain::radtreeb_changed( bool val )
{
	if ( !val ) return;

	layout_state = 0;

	renderer_canvas->setLayout( layout_state );
}

void QMain::invradtreeb_changed( bool val )
{
	if ( !val ) return;

	layout_state = 1;

	renderer_canvas->setLayout( layout_state );
}

void QMain::roottreeb_changed( bool val )
{
	if ( !val ) return;

	layout_state = 2;

	renderer_canvas->setLayout( layout_state );
}

void QMain::invroottreeb_changed( bool val )
{
	if ( !val ) return;

	layout_state = 3;

	renderer_canvas->setLayout( layout_state );
}

void QMain::baltreeb_changed( bool val )
{
	if ( !val ) return;

	layout_state = 4;

	m_Tree.computeBalloon();
	renderer_canvas->setLayout( layout_state );
}

void QMain::betaval_changed( int val )
{
	m_Tree.setBeta( (float) val / 100.0f );

	//update the gui
	char buffer[16];
	int precoma = val / 100;
	int postcoma_2 = ( ( val - precoma * 100 ) * 100 ) / 100;

	sprintf_s(buffer, "%d.%d", precoma, postcoma_2);
	beta_text->setText(buffer);

	renderer_canvas->repaint();
}

void QMain::emphval_changed( int val )
{
	m_Tree.setEmphasize( (float) val / 100.0f );

	//update the gui
	char buffer[16];
	int precoma = val / 100;
	int postcoma_2 = ( ( val - precoma * 100 ) * 100 ) / 100;

	sprintf_s(buffer, "%d.%d", precoma, postcoma_2);
	emphasize_text->setText(buffer);

	renderer_canvas->repaint();
}

void QMain::alphaval_changed( int val )
{
	m_Tree.setAlpha( (float) val / 100.0f );

	//update the gui
	char buffer[16];
	int precoma = val / 100;
	int postcoma_2 = ( ( val - precoma * 100 ) * 100 ) / 100;

	sprintf_s(buffer, "%d.%d", precoma, postcoma_2);
	alpha_text->setText(buffer);

	renderer_canvas->repaint();
}

void QMain::select_splines( bool val )
{
	splines_selstate = val;

}