#include "QVolRendCanvas.h"

#include "devIl/IL/il.h"
#include "devIl/IL/ilu.h"
#include "devIl/IL/ilut.h"

bool isfirstframe = true;
bool placingpoint_enabled = false;
bool selectionpoint_enabled = false;
bool drawingline_enabled = false;
int num_setpoints = 0;
VMouseEvent global_me;

VVector getPlaneProjection(float x, float y)
{
	VVector vecResult;
	vecResult.setX(x);
	vecResult.setY(y);
	vecResult.setZ(0.0f);

	return vecResult;
}

QVolRendCanvas::QVolRendCanvas( QWidget* parent /*= 0*/ ) : 
			QObject( parent ), 
			QGLWidget( parent ),
			m_PanOffset( 0.0f, 0.0f, 0.0f ),
			m_ZoomFactor( 1.0f ),
			m_LineStart( 0.0f, 0.0f, 0.0f ),
			m_LineEnd( 0.0f, 0.0f, 0.0f ),
			name( NULL ), born( NULL ), died( NULL ), foto( NULL ), parents( NULL ), kids( NULL )
{

	glInit();

	ilInit();
	iluInit();
	ilutInit();
	ilutRenderer(ILUT_OPENGL);

	if ((ilGetInteger(IL_VERSION_NUM) < IL_VERSION) ||
		(iluGetInteger(ILU_VERSION_NUM) < ILU_VERSION) ||
		(ilutGetInteger(ILUT_VERSION_NUM) < ILUT_VERSION)) 
	{
		std::cout << "devil not working" << std::endl;
		std::cout << "Press a button to continue" << std::endl;
		std::cin.get();
		exit(-1);
		return;
	}

	tLayout = LAY_RADIAL;
	m_MouseState = NO_DRAGGING;
}


QVolRendCanvas::~QVolRendCanvas()
{
	
//	m_BackgroundColor = NULL;

	if ( m_TreeRep ) 
	{
		//delete m_TreeRep;
		m_TreeRep = NULL;
	}

	
}

void QVolRendCanvas::setObjectName( const QString &name )
{
}



void QVolRendCanvas::setBackColorPtr( VVector *v )
{
	m_BackgroundColor = v;
}
void QVolRendCanvas::initializeGL()
{
	this->makeCurrent();
    // Set up the rendering context, define display lists etc.:
    //...
	glClearColor( 0.0, 0.0, 0.0, 1.0 );
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    glEnable( GL_DEPTH_TEST );
    //...
}

void QVolRendCanvas::resizeGL( int w, int h )
{
	this->makeCurrent();
	glViewport( 0, 0, w, h );

	
	/*if(m_FramebufferObject)
	{
		delete m_FramebufferObject;
		m_FramebufferObject = NULL;
	}

	m_FramebufferObject = new VFramebufferObject(width(), height(), GL_RGBA);
	m_FramebufferObject->init();*/

	//m_VolumeRenderer.resize(w, h);
}




void QVolRendCanvas::paintGL()
{	
	/*this*/makeCurrent();


	glClearColor( m_BackgroundColor->getX() , m_BackgroundColor->getY() , m_BackgroundColor->getZ() , 1.0f );
	glClearDepth( -1000.0f );

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// Clear Screen And Depth Buffer
    glEnable( GL_DEPTH_TEST );

	glEnable( GL_BLEND );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	glEnable( GL_DEPTH_TEST );
	glDepthFunc( GL_ALWAYS );

    // draw the scene:
    glPushAttrib( GL_ALL_ATTRIB_BITS );
	
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	
	glLoadIdentity();									// Reset The Current Modelview Matrix

	glTranslatef( m_PanOffset.getX(), m_PanOffset.getY(), 0.0f );
	glScalef( m_ZoomFactor, m_ZoomFactor, 1.0f );

	m_TreeRep->setScale( m_ZoomFactor );

	float line[ 4 ];
	line[ 0 ] = m_LineStart.getX();
	line[ 1 ] = m_LineStart.getY();
	line[ 2 ] = m_LineEnd.getX();
	line[ 3 ] = m_LineEnd.getY();
	bool selectiveopacity = drawingline_enabled && ( num_setpoints > 0 );

	switch ( tLayout )
	{
		case LAY_RADIAL:
		{
			glEnable(GL_DEPTH_TEST);
			m_TreeRep->drawRadial();

			glDisable(GL_DEPTH_TEST);
			if( num_setpoints != 2 )
			{
				if( *m_splinesel_active )
				{
					m_TreeRep->hasIntersections( selectiveopacity, line, m_ZoomFactor, &m_PanOffset );
				}
				else
				{
					m_TreeRep->hasIntersections( false, line, m_ZoomFactor, &m_PanOffset );
				}
			}
			else if( ! *m_splinesel_active )
			{
				m_TreeRep->hasIntersections( false, line, m_ZoomFactor, &m_PanOffset );
			}

			m_TreeRep->drawSplinesNormal();
			
			glEnable(GL_DEPTH_TEST);
			m_TreeRep->drawRadial();
			
			//m_TreeRep->m_bUpdateSplines = true;
			break;
		}
		case LAY_INVRADIAL:
		{
			if( num_setpoints != 2 )
			{
				if( *m_splinesel_active )
				{
					m_TreeRep->hasIntersections( selectiveopacity, line, m_ZoomFactor, &m_PanOffset );
				}
				else
				{
					m_TreeRep->hasIntersections( false, line, m_ZoomFactor, &m_PanOffset );
				}
			}
			else if( ! *m_splinesel_active )
			{
				m_TreeRep->hasIntersections( false, line, m_ZoomFactor, &m_PanOffset );
			}

			//glDisable(GL_DEPTH_TEST);
			m_TreeRep->drawSplinesNormal();


			glEnable(GL_DEPTH_TEST);
			m_TreeRep->drawInvRadial();
			
			//m_TreeRep->m_bUpdateSplines = true;
			break;
		}
		case LAY_ROOTED:
		{
			if( num_setpoints != 2 )
			{
				if( *m_splinesel_active )
				{
					m_TreeRep->hasIntersections( selectiveopacity, line, m_ZoomFactor, &m_PanOffset );
				}
				else
				{
					m_TreeRep->hasIntersections( false, line, m_ZoomFactor, &m_PanOffset );
				}
			}
			else if( ! *m_splinesel_active )
			{
				m_TreeRep->hasIntersections( false, line, m_ZoomFactor, &m_PanOffset );
			}

			//glDisable(GL_DEPTH_TEST);
			m_TreeRep->drawSplinesNormal();
			glEnable(GL_DEPTH_TEST);
			m_TreeRep->drawRooted();
			
			//m_TreeRep->m_bUpdateSplines = true;
			break;
		}
		case LAY_INVROOTED:
		{
			/*glEnable(GL_DEPTH_TEST);
			m_TreeRep->drawInvRooted();*/

			if( num_setpoints != 2 )
			{
				if( *m_splinesel_active )
				{
					m_TreeRep->hasIntersectionsSpecial( selectiveopacity, line, m_ZoomFactor, &m_PanOffset );
				}
				else
				{
					m_TreeRep->hasIntersectionsSpecial( false, line, m_ZoomFactor, &m_PanOffset );
				}
			}
			else if( ! *m_splinesel_active )
			{
				m_TreeRep->hasIntersectionsSpecial( false, line, m_ZoomFactor, &m_PanOffset );
			}

			glDisable(GL_DEPTH_TEST);
			m_TreeRep->drawSplinesSpecial();

			glEnable(GL_DEPTH_TEST);
			m_TreeRep->drawInvRooted();
			
			//m_TreeRep->m_bUpdateSplines = true;
			break;
		}
		case LAY_BALLOON:
		{
			glEnable(GL_DEPTH_TEST);
			//m_TreeRep->computeBalloon();
			m_TreeRep->drawBalloon();

			if( num_setpoints != 2 )
			{
				if( *m_splinesel_active )
				{
					m_TreeRep->hasIntersections( selectiveopacity, line, m_ZoomFactor, &m_PanOffset );
				}
				else
				{
					m_TreeRep->hasIntersections( false, line, m_ZoomFactor, &m_PanOffset );
				}
			}
			else if( ! *m_splinesel_active )
			{
				m_TreeRep->hasIntersections( false, line, m_ZoomFactor, &m_PanOffset );
			}

			glDisable(GL_DEPTH_TEST);
			m_TreeRep->drawSplinesNormal();

			glEnable(GL_DEPTH_TEST);
			m_TreeRep->drawBalloon();
			
			//m_TreeRep->m_bUpdateSplines = true;
			break;
		}
	}

	glLoadIdentity();

	if ( drawingline_enabled && ( num_setpoints > 0 ) )
	{
		glEnable( GL_DEPTH_TEST );
		glDepthFunc( GL_ALWAYS );

		glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
		glBegin( GL_LINES );
			glVertex3fv( m_LineStart.getPtr() );
			glVertex3fv( m_LineEnd.getPtr() );
		glEnd();

		glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
		glBegin( GL_POINTS );
			glVertex3fv( m_LineStart.getPtr() );
			glVertex3fv( m_LineEnd.getPtr() );
		glEnd();

	}	

	if( selectionpoint_enabled )
	{
		glEnable( GL_DEPTH_TEST );
		glDepthFunc( GL_ALWAYS );
		
		VVector pos = global_me.getPosition();

		glColor4f( 241.0f / 255.0f, 32.0f / 255.0f, 126.0f / 255.0f, 1.0f );
		glBegin( GL_POINTS );
			glVertex3fv( pos.getPtr() );
		glEnd();

	}

	GLenum err = glGetError(); 

	if (err != GL_NO_ERROR) 
	{ 

		std::cout << "glError in slice based renderer: " << gluErrorString(err) << std::endl;
	} 

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glPopAttrib();

}

void QVolRendCanvas::paintEvent( QPaintEvent *e )
{
	QGLWidget::paintEvent(e);
};

void QVolRendCanvas::mousePressEvent ( QMouseEvent * e )
{		
	const VMouseEvent me(getMouseEvent(e));

	if( me.getModifiers() == me.MODIFIER_CTRL )
	{
		if( num_setpoints == 0 )
		{
			m_LineStart = me.getPosition();
			num_setpoints++;
			drawingline_enabled = false;
		}
		else if ( num_setpoints == 1 )
		{
			m_LineEnd = me.getPosition();
			num_setpoints++;
		}else 
		{
			m_LineStart = me.getPosition();
			num_setpoints = 1;
			drawingline_enabled = false;
		}
		return;
	}
	if( me.getModifiers() == me.MODIFIER_SHIFT )
	{
		VVector pos = me.getPosition();

		pos.setX( (	pos.getX() - m_PanOffset.getX() ) / m_ZoomFactor );
		pos.setY( ( pos.getY() - m_PanOffset.getY() ) / m_ZoomFactor );
		pos.setZ( ( pos.getZ() - m_PanOffset.getZ() ) / m_ZoomFactor );

		bool success;
		tree<TreeEntity>::iterator te = m_TreeRep->find( pos.getX(), pos.getY(), &success );

		if( success )
		{
			tree<TreeEntity>::iterator parent = te.node->parent;
			tree<TreeEntity>::iterator child1 = te.node->first_child;
			tree<TreeEntity>::iterator child2 = te.node->last_child;

			name->setText( (te->name).c_str() );
			born->setText( (te->born).c_str() );
			died->setText( (te->died).c_str() );
			foto->setTexHandlePtr( &( te->pic_handle ) );
			foto->repaint();

			if( parent.node )
			{
				kids->setText( (parent->name).c_str() );
			}
			else
			{
				kids->setText( "" );
			}

			char buffer1[128];
			char buffer2[128];
			char buffer[256];
			if( child1.node )
			{
				
				sprintf_s( buffer1, "%s\n", (child1->name).c_str() ); 
			}

			if( child2.node )
			{
				sprintf_s( buffer2, "%s", (child2->name).c_str() ); 
			}

			if( child1.node || child2.node )
			{
				sprintf_s( buffer, "%s%s", buffer1, buffer2 );
				parents->setText( buffer );
			}
			else
			{
				parents->setText( "" );
			}
			
		}

		return;
		
	}

	setCursor(QCursor(Qt::CursorShape::ClosedHandCursor));

	float x = me.getPosition().getX();
	float y = me.getPosition().getY();

	if(me.getRightState())
	{
		m_MouseState = ZOOM;
		m_vecPosition = getPlaneProjection(x,y);
		m_vecPosition *= VVector((float)height(), (float)width(), 0.0f);
	}
	else if(me.getMiddleState())
	{
		m_MouseState = PAN;
		m_vecPosition = getPlaneProjection(x,y);
	}
	
};

void QVolRendCanvas::mouseReleaseEvent ( QMouseEvent * e )
{
	const VMouseEvent me(getMouseEvent(e));

	if( me.getModifiers() != me.MODIFIER_CTRL )
	{
		setCursor(QCursor(Qt::CursorShape::OpenHandCursor));
	}

	m_MouseState = NO_DRAGGING;
	isfirstframe = true;
	
};
	
void QVolRendCanvas::mouseMoveEvent ( QMouseEvent * e )
{
	

	const VMouseEvent me(getMouseEvent(e));
	global_me = me;

	if ( me.getModifiers() == me.MODIFIER_CTRL && num_setpoints == 1 )
	{
		m_LineEnd = me.getPosition();
		drawingline_enabled = true;
		return;
	}

	// v is the position of the mouse
	// cursor over canvas: v ranges in R, from -1 to 1 in both directions, left bottom corner is -1,-1
	// cursor n.o. canvas: if mouse was pressed before cursor left canvas, position is outside the range R 
	const VVector v  = me.getPosition();


	float x = v.getX();
	float y = v.getY();

	float ny = 0.0f;
	float nx = 0.0f;

	switch (m_MouseState)
	{

	case PAN:
		{
			const VVector vecPosition = getPlaneProjection(x,y);

			if( isfirstframe )
			{
				m_vecPosition = vecPosition;
				isfirstframe = false;
				break;
			}
			
			const VVector vecDifference = vecPosition - m_vecPosition;
			
			m_PanOffset += vecDifference / 2.0f;

			m_vecPosition = vecPosition;
		}
		break;

	case ZOOM:
		{
			const VVector vecPosition = getPlaneProjection(x,y);

			if( isfirstframe )
			{
				m_vecPosition = vecPosition;
				isfirstframe = false;
				break;
			}

			VVector vecDifference = vecPosition - m_vecPosition;

			vecDifference.setX(vecDifference.getX());
			vecDifference.setY(-vecDifference.getY());
			vecDifference.setZ(0.0);

			float offset = (vecDifference.getY() > 0.0) ? sqrt(vecDifference.getDot(vecDifference)) : -sqrt(vecDifference.getDot(vecDifference));
			m_ZoomFactor *= exp( -offset * log( 1.5 ) );

			m_vecPosition = vecPosition;
		}
		break;
	}
};

void QVolRendCanvas::mouseDoubleClickEvent ( QMouseEvent * e )
{
	
};

void QVolRendCanvas::keyPressEvent ( QKeyEvent * e )
{
	const VKeyboardEvent ke(getKeyboardEvent(e));

	if( ke.getModifiers() == ke.MODIFIER_CTRL )
	{
		setCursor(QCursor(Qt::CursorShape::CrossCursor));
		placingpoint_enabled = true;

		// if the line is already wholy set (start/end are well-defined), I can switch it on by pressing ctrl
		if( num_setpoints > 1 )
		{
			drawingline_enabled = true;
		}else
		{
			drawingline_enabled = false;
			num_setpoints = 0;
		}
		

	}

	if( ke.getModifiers() == ke.MODIFIER_SHIFT )
	{
		setCursor(QCursor(Qt::CursorShape::CrossCursor));
		selectionpoint_enabled = true;

	}

};

void QVolRendCanvas::keyReleaseEvent ( QKeyEvent * e )
{
	setCursor(QCursor(Qt::CursorShape::OpenHandCursor));
	placingpoint_enabled = false;
	drawingline_enabled = false;
	selectionpoint_enabled = false;
	//num_setpoints = 0;
	
};

const VMouseEvent QVolRendCanvas::getMouseEvent (QMouseEvent *e)
{
	const VVector vecPosition( (2.0f * float(e->x()) - float(width()))  / float(width()),
		                      ( float(height()) - 2.0f * float(e->y())) / float(height()),
							   0.0f);
	const int iButton = 
				((e->button() == Qt::LeftButton)	? VMouseEvent::BUTTON_LEFT	 : 
				((e->button() == Qt::MidButton)		? VMouseEvent::BUTTON_MIDDLE : 
				((e->button() == Qt::RightButton)	? VMouseEvent::BUTTON_RIGHT	 : VMouseEvent::BUTTON_NONE)));

	const int iStateLeft	= (e->buttons () & Qt::LeftButton)	? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;
	const int iStateMiddle	= (e->buttons () & Qt::MidButton)	? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;
	const int iStateRight	= (e->buttons () & Qt::RightButton) ? VMouseEvent::STATE_DOWN : VMouseEvent::STATE_UP;

	const int iModifiers =   ((e->modifiers()   & Qt::ShiftModifier)   ? VKeyboardEvent::MODIFIER_SHIFT : 0) 
						   | ((e->modifiers()	& Qt::ControlModifier) ? VKeyboardEvent::MODIFIER_CTRL  : 0) 
						   | ((e->modifiers()	& Qt::AltModifier)     ? VKeyboardEvent::MODIFIER_ALT   : 0);

	return VMouseEvent(vecPosition,iButton,iStateLeft,iStateMiddle,iStateRight,iModifiers);

};

const VKeyboardEvent QVolRendCanvas::getKeyboardEvent (QKeyEvent *e)
{
	const int iKey = e->key();

	const int iModifiers =	  ((e->modifiers() & Qt::ShiftModifier)		? VKeyboardEvent::MODIFIER_SHIFT : 0) 
							| ((e->modifiers() & Qt::ControlModifier)	? VKeyboardEvent::MODIFIER_CTRL	 : 0) 
							| ((e->modifiers() & Qt::AltModifier)		? VKeyboardEvent::MODIFIER_ALT	 : 0);

	return VKeyboardEvent(iKey,iModifiers);

};

void QVolRendCanvas::redraw()
{
	/*this*/makeCurrent();
	paintGL();
}

void QVolRendCanvas::setLayout( int layout_state )
{
	switch (layout_state)
	{
		case 0:
		{
			tLayout = LAY_RADIAL;
			break;
		}
		case 1:
		{
			tLayout = LAY_INVRADIAL;
			break;
		}
		case 2:
		{
			tLayout = LAY_ROOTED;
			break;
		}
		case 3:
		{
			tLayout = LAY_INVROOTED;
			break;
		}
		case 4:
		{
			tLayout = LAY_BALLOON;
			break;
		}
	}
}

int QVolRendCanvas::getLayout()
{
	switch (tLayout)
	{
		case LAY_RADIAL:
		{
			return 0;
		}
		case LAY_INVRADIAL:
		{
			return 1;
		}
		case LAY_ROOTED:
		{
			return 2;
		}
		case LAY_INVROOTED:
		{
			return 3;
		}
		case LAY_BALLOON:
		{
			return 4;
		}
	}
}
